import concurrent.futures
import logging

from spaceone.core.service import *
from spaceone.core.utils import get_dict_value, random_string

from spaceone.monitoring.conf.global_conf import *
from spaceone.monitoring.error import *
from spaceone.monitoring.manager.data_source_manager import DataSourceManager
from spaceone.monitoring.manager.data_source_plugin_manager import (
    DataSourcePluginManager,
)
from spaceone.monitoring.manager.inventory_manager import InventoryManager
from spaceone.monitoring.manager.secret_manager import SecretManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class MetricService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.inventory_mgr: InventoryManager = self.locator.get_manager(
            "InventoryManager"
        )
        self.secret_mgr: SecretManager = self.locator.get_manager("SecretManager")
        self.data_source_mgr: DataSourceManager = self.locator.get_manager(
            "DataSourceManager"
        )
        self.ds_plugin_mgr: DataSourcePluginManager = self.locator.get_manager(
            "DataSourcePluginManager"
        )

    @transaction(
        permission="monitoring:Metric.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["data_source_id", "resources", "domain_id"])
    def list(self, params):
        """Get resource's metrics

        Args:
            params (dict): {
                'data_source_id': 'str',   # required
                'resources': 'list',       # required
                'domain_id': 'str'         # injected from auth (required)
            }

        Returns:
            metrics (list)
        """
        data_source_id = params["data_source_id"]
        resources = params["resources"]
        domain_id = params["domain_id"]

        data_source_vo = self.data_source_mgr.get_data_source(data_source_id, domain_id)
        self._check_data_source_state(data_source_vo)

        plugin_info = data_source_vo.plugin_info
        plugin_metadata = plugin_info.metadata
        self._check_plugin_metadata(plugin_metadata, params, data_source_id)
        self.plugin_initialize(data_source_vo)

        metrics_dict = {}
        and_metric_keys = []

        response = {"metrics": [], "available_resources": {}, "domain_id": domain_id}

        for resource_id in resources:
            response["available_resources"][resource_id] = False

        required_keys = plugin_metadata.get("required_keys")
        cloud_services_info = self.inventory_mgr.list_resources(
            resources, required_keys, domain_id
        )

        list_metric_params = []
        for cloud_service_info in cloud_services_info:
            secret = self.secret_mgr.get_secret_from_resource(
                cloud_service_info, data_source_vo, domain_id
            )
            secret_data = self.secret_mgr.get_secret_data(
                secret.get("secret_id", ""), domain_id
            )

            query = self.get_query_from_cloud_service(cloud_service_info, plugin_info)

            list_metric_params.append(
                {
                    "schema": secret.get("schema_id"),
                    "options": plugin_info.options,
                    "secret_data": secret_data,
                    "query": query,
                    "cloud_service_id": cloud_service_info["cloud_service_id"],
                    "data_source_vo": data_source_vo,
                    "domain_id": domain_id,
                    "metrics_dict": metrics_dict,
                    "and_metric_keys": and_metric_keys,
                }
            )

        metric_responses = []
        with concurrent.futures.ThreadPoolExecutor(
            max_workers=MAX_CONCURRENT_WORKER
        ) as executor:
            future_executors = []

            for _params in list_metric_params:
                future_executors.append(
                    executor.submit(self.list_metrics_info, _params)
                )

            for future in concurrent.futures.as_completed(future_executors):
                metric_responses.append(future.result())

        response = self._merge_metric(metric_responses, domain_id)
        return response

    @transaction(
        permission="monitoring:Metric.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(
        ["data_source_id", "metric_query", "metric", "start", "end", "domain_id"]
    )
    def get_data(self, params):
        """Get resource's metric data

        Args:
            params (dict): {
                'data_source_id': 'str',    # required
                'metric_query': 'dict',     # required
                'metric': 'str',            # required
                'start': 'str',             # required
                'end': 'str',               # required
                'period': 'int',
                'stat': 'str',
                'domain_id': 'str'          # injected from auth (required)
            }

        Returns:
            metric_data (list)
        """
        data_source_id = params["data_source_id"]
        metric_query = params["metric_query"]
        domain_id = params["domain_id"]

        data_source_vo = self.data_source_mgr.get_data_source(data_source_id, domain_id)
        self._check_data_source_state(data_source_vo)

        plugin_metadata = data_source_vo.plugin_info.metadata
        self._check_plugin_metadata(plugin_metadata, params, data_source_id)
        self.plugin_initialize(data_source_vo)

        response = {"labels": [], "values": {}, "domain_id": domain_id}

        required_keys = plugin_metadata.get("required_keys")
        resource_ids = self.get_resource_ids_from_metric_query(metric_query)
        resources_info = self.inventory_mgr.list_resources(
            resource_ids, required_keys, domain_id
        )
        resources_chunks = self.list_chunk_resources(
            resources_info, metric_query, data_source_vo, domain_id
        )

        _LOGGER.debug(f"[get_data] chunk_resources: {resources_chunks}")

        metric_data_params = self.set_metric_data_params(params)

        for chunk_resources in resources_chunks.values():
            metric_data_params.update(
                {
                    "secret_data": chunk_resources.get("secret_data"),
                    "metric_query": chunk_resources.get("metric_query"),
                    "options": chunk_resources.get("options", {}),
                }
            )

            if "schema" in chunk_resources:
                metric_data_params.update({"schema": chunk_resources.get("schema")})

            metric_data_response = self.get_metric_data(metric_data_params)

            if not response["labels"] and metric_data_response.get("labels", []):
                response["labels"] = metric_data_response["labels"]

            if values := metric_data_response.get("values"):
                response["values"].update(values)

        return response

    def list_chunk_resources(self, resources, metric_query, data_source_vo, domain_id):
        """
        chunk_resources(dict): {
            'provider.region_code.secret_id.random_key': {
                'schema': 'str',
                'secret_data': 'dict',
                'region_name': 'str',
                'options': 'dict',
                'metric_query': 'dict'     # MAX_REQUEST_LIMIT
            },
            ...
        }
        """
        chunk_resources = {}
        provider = data_source_vo.plugin_info["provider"]

        for resource in resources:
            cloud_service_id = resource["cloud_service_id"]
            if cloud_service_id in metric_query:
                region_code = self.get_region_from_resource(resource)
                secret = self.secret_mgr.get_secret_from_resource(
                    resource, data_source_vo, domain_id
                )
                chunk_key = self._generate_chunk_key(
                    provider, region_code, secret.get("secret_id", "")
                )

                metric_query[cloud_service_id]["region_name"] = region_code
                chunk_resource = {cloud_service_id: metric_query[cloud_service_id]}

                if chunk_key in chunk_resources:
                    chunk_info = chunk_resources[chunk_key]

                    if "region_name" not in chunk_info:
                        chunk_info.update({"region_name": region_code})

                    if "secret_data" not in chunk_info:
                        chunk_info.update(
                            {
                                "secret_data": self.secret_mgr.get_secret_data(
                                    secret["secret_id"], domain_id
                                )
                            }
                        )

                    if "schema" not in chunk_info:
                        chunk_info.update({"schema": secret.get("schema_id")})

                    _metric_query = chunk_info["metric_query"]

                    if len(_metric_query) >= MAX_REQUEST_LIMIT:
                        new_chunk_key = f"{chunk_key}.{random_string(1)}"
                        # move chunk to new key
                        chunk_resources[new_chunk_key] = chunk_info
                        # generate a new chunk
                        chunk_resources[chunk_key] = {
                            "region_name": region_code,
                            "secret_data": self.secret_mgr.get_secret_data(
                                secret["secret_id"], domain_id
                            ),
                            "metric_query": chunk_resource,
                        }
                    else:
                        _metric_query[cloud_service_id] = metric_query[cloud_service_id]
                else:
                    # generate a new chunk
                    chunk_resources[chunk_key] = {
                        "region_name": region_code,
                        "secret_data": self.secret_mgr.get_secret_data(
                            secret["secret_id"], domain_id
                        ),
                        "metric_query": chunk_resource,
                    }

        return chunk_resources

    def plugin_initialize(self, data_source_vo):
        endpoint = self.ds_plugin_mgr.get_data_source_plugin_endpoint_by_vo(
            data_source_vo
        )
        self.ds_plugin_mgr.initialize(endpoint)

    def get_metric_data(self, params):
        metric_data_info = {"labels": [], "values": {}}

        try:
            metric_data_info = self.ds_plugin_mgr.get_metric_data(params)
        except Exception as e:
            print(e)

        return metric_data_info

    def list_metrics_info(self, params):
        schema = params.get("schema")
        secret_data = params["secret_data"]
        cloud_service_id = params["cloud_service_id"]
        query = params.get("query", {})
        options = params.get("options", {})

        try:
            metrics_info = self.ds_plugin_mgr.list_metrics(
                schema, options, secret_data, query
            )
            _LOGGER.debug(f"[list_metric_info] metrics_info: {metrics_info}")
            return {
                "metrics": metrics_info.get("metrics", []),
                "resource_id": cloud_service_id,
            }
            # return response
        except Exception as e:
            _LOGGER.error(f"[list_metrics]: {e}")
            return {}

    def _merge_metric(self, metric_responses, domain_id):
        response = {"metrics": [], "available_resources": {}, "domain_id": domain_id}

        (
            available_resources,
            metric_responses,
            cloud_service_metric_set_info,
        ) = self.set_preload_metric(metric_responses)

        _LOGGER.debug(f"available_resources: {available_resources}")
        _LOGGER.debug(f"metric_responses: {metric_responses}")
        _LOGGER.debug(f"cloud_service_metric_set_info: {cloud_service_metric_set_info}")

        if len(metric_responses) == 1:
            response["metrics"] = metric_responses[0]["metrics"]
        elif len(metric_responses) > 1:
            _intersected_metric_keys = self._intersection_metric_keys(
                cloud_service_metric_set_info
            )

            response_metrics = []
            for metric_response in metric_responses:
                for _metric in metric_response["metrics"]:
                    if _metric["key"] in _intersected_metric_keys:
                        match_key = False
                        for response_metric in response_metrics:
                            if response_metric["key"] == _metric["key"]:
                                response_metric["metric_query"].update(
                                    _metric["metric_query"]
                                )
                                match_key = True
                                break

                        if match_key is False:
                            response_metrics.append(_metric)

            response["metrics"] = response_metrics

        response["available_resources"] = available_resources

        _LOGGER.debug(response)
        return response

    @staticmethod
    def set_preload_metric(metric_responses):
        available_resources = {}
        cloud_service_metric_set_info = {}

        for metric_response in metric_responses:
            if cloud_service_id := metric_response.get("resource_id"):
                available_resources[cloud_service_id] = True

                _metric_keys = []
                for _metric in metric_response["metrics"]:
                    _metric_keys.append(_metric["key"])

                    _metric_query = _metric["metric_query"]
                    _metric["metric_query"] = {cloud_service_id: _metric_query}

                cloud_service_metric_set_info[cloud_service_id] = _metric_keys

        return available_resources, metric_responses, cloud_service_metric_set_info

    @staticmethod
    def _intersection_metric_keys(cloud_service_metric_set_info):
        target_metrics = []

        for metrics in cloud_service_metric_set_info.values():
            if metrics and target_metrics:
                target_metrics = list(set(metrics) & set(target_metrics))

            if metrics and not target_metrics:
                target_metrics = metrics

        return target_metrics

    @staticmethod
    def get_resource_ids_from_metric_query(metric_query):
        return [resource_id for resource_id in metric_query.keys()]

    @staticmethod
    def set_metric_data_params(params):
        return_params = {
            "metric": params["metric"],
            "start": params["start"],
            "end": params["end"],
        }

        if "period" in params:
            return_params["period"] = params["period"]

        if "stat" in params:
            return_params["stat"] = params["stat"]

        return return_params

    @staticmethod
    def _generate_chunk_key(provider, region_code, secret_id):
        return f"{provider}.{region_code}.{secret_id}"

    @staticmethod
    def get_region_from_resource(resource):
        return resource.get("region_code", "")

    @staticmethod
    def get_account_from_resource(resource):
        return resource.get("account", "")

    @staticmethod
    def _check_data_source_state(data_source_vo):
        if data_source_vo.state == "DISABLED":
            raise ERROR_DATA_SOURCE_STATE_DISABLED(
                data_source_id=data_source_vo.data_source_id
            )

    @staticmethod
    def _check_resource_type(plugin_metadata, resource_type):
        supported_resource_type = plugin_metadata["supported_resource_type"]

        if resource_type not in supported_resource_type:
            raise ERROR_NOT_SUPPORT_RESOURCE_TYPE(
                supported_resource_type=supported_resource_type
            )

    @staticmethod
    def _check_plugin_metadata(metadata, params, data_source_id):
        if "supported_stat" in metadata and "stat" in params:
            if params["stat"] not in metadata["supported_stat"]:
                raise ERROR_NOT_SUPPORT_METRIC_STAT(stat=params["stat"])

        if "required_keys" in metadata:
            if not metadata["required_keys"]:
                raise ERROR_NOT_FOUND_REQUIRED_KEY(data_source_id=data_source_id)
        else:
            raise ERROR_NOT_FOUND_REQUIRED_KEY(data_source_id=data_source_id)

    @staticmethod
    def _merge_metric_keys(metrics_info, metrics_dict, and_metric_keys):
        for metric_info in metrics_info.get("metrics", []):
            if "key" in metric_info:
                metric_key = metric_info["key"]

                if metric_key not in and_metric_keys:
                    metrics_dict[metric_key] = metric_info
                    and_metric_keys.append(metric_key)

        return {"metrics_dict": metrics_dict, "and_metric_keys": and_metric_keys}

    @staticmethod
    def get_required_key(metadata):
        required_keys = metadata.get("required_keys", [])

        if not required_keys:
            raise

        return required_keys[0]

    @staticmethod
    def get_query_from_cloud_service(cloud_service_info, plugin_info):
        metadata = plugin_info.metadata
        required_keys = metadata.get("required_keys", [])

        if required_keys:
            query_key = required_keys[0]
            return get_dict_value(cloud_service_info, query_key, default_value={})
        else:
            raise ERROR_REQUIRED_KEYS_NOT_EXISTS(plugin_id=plugin_info.plugin_id)



import logging
from datetime import timedelta, datetime
from typing import Union

from spaceone.core import cache, config, utils
from spaceone.core.service import *

from spaceone.monitoring.manager.alert_manager import AlertManager
from spaceone.monitoring.manager.escalation_policy_manager import (
    EscalationPolicyManager,
)
from spaceone.monitoring.manager.identity_manager import IdentityManager
from spaceone.monitoring.manager.job_manager import JobManager
from spaceone.monitoring.manager.notification_manager import NotificationManager
from spaceone.monitoring.manager.project_alert_config_manager import (
    ProjectAlertConfigManager,
)
from spaceone.monitoring.manager.webhook_manager import WebhookManager
from spaceone.monitoring.model.alert_model import Alert
from spaceone.monitoring.model.escalation_policy_model import EscalationPolicy
from spaceone.monitoring.model.project_alert_config_model import ProjectAlertConfig

_LOGGER = logging.getLogger(__name__)


@event_handler
class JobService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.job_mgr: JobManager = self.locator.get_manager("JobManager")

    @transaction
    def create_jobs_by_domain(self, params):
        """Create jobs by domain

        Args:
            params (dict): {}

        Returns:
            None
        """

        for domain_id in self._list_domains_of_alerts():
            _LOGGER.debug(
                f"[create_jobs_by_domain] Push task (JobService.create): {domain_id}"
            )
            self.job_mgr.push_task(
                "monitoring_alert_job",
                "JobService",
                "create_job",
                {"domain_id": domain_id},
            )

    @transaction
    @check_required(["domain_id"])
    def create_job(self, params):
        """Create job

        Args:
            params (dict): {
                'domain_id': 'str'
            }

        Returns:
            None
        """

        domain_id = params["domain_id"]

        if self.job_mgr.is_domain_job_running(domain_id):
            _LOGGER.debug(f"[create_job] Job is running. (domain_id = {domain_id})")
            return None

        job_vo = self.job_mgr.create_job(domain_id)
        try:
            alert_vos, total_count = self._list_open_alerts(domain_id)

            if total_count > 0:
                job_vo.update(
                    {"total_tasks": total_count, "remained_tasks": total_count}
                )

                for alert_vo in alert_vos:
                    _LOGGER.debug(
                        f"[create_job] Push task (JobService.create_notification): {alert_vo.alert_id}"
                    )
                    self.job_mgr.push_task(
                        "monitoring_alert_notification_from_scheduler",
                        "JobService",
                        "create_alert_notification",
                        {
                            "job_id": job_vo.job_id,
                            "alert_id": alert_vo.alert_id,
                            "domain_id": alert_vo.domain_id,
                        },
                    )
            else:
                job_vo.delete()
        except Exception as e:
            self.job_mgr.change_error_status(job_vo, e)
            self.transaction.execute_rollback()

    @transaction
    @check_required(["alert_id", "domain_id"])
    def create_assigned_notification(self, params):
        """Create assigned notification

        Args:
            params (dict): {
                'alert_id': 'str',
                'domain_id': 'str',
                'user_id': 'str'
            }

        Returns:
            None
        """

        alert_id = params["alert_id"]
        domain_id = params["domain_id"]
        user_id = params["user_id"]

        alert_mgr: AlertManager = self.locator.get_manager("AlertManager")

        alert_vo: Alert = alert_mgr.get_alert(alert_id, domain_id)

        title = f"[Assigned to me] {alert_vo.title}"

        notification_mgr: NotificationManager = self.locator.get_manager(
            "NotificationManager"
        )
        message = self._create_message(alert_vo, title, "INFO", user_id=user_id)

        notification_mgr.create_notification(message)

    @transaction
    @check_required(["alert_id", "domain_id"])
    def create_resolved_notification(self, params):
        """Create resolved notification

        Args:
            params (dict): {
                'alert_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """

        alert_id = params["alert_id"]
        domain_id = params["domain_id"]

        alert_mgr: AlertManager = self.locator.get_manager("AlertManager")

        alert_vo: Alert = alert_mgr.get_alert(alert_id, domain_id)
        project_id = alert_vo.project_id
        escalation_policy_id = alert_vo.escalation_policy_id

        (
            rules,
            finish_condition,
        ) = self._get_escalation_policy_rules_and_finish_condition(
            escalation_policy_id, domain_id
        )

        title = f"[Resolved] {alert_vo.title}"

        for step in range(alert_vo.escalation_step):
            notification_level = rules[step]["notification_level"]

            notification_mgr: NotificationManager = self.locator.get_manager(
                "NotificationManager"
            )
            message = self._create_message(
                alert_vo, title, "SUCCESS", notification_level=notification_level
            )

            notification_mgr.create_notification(message)

    @transaction
    @check_required(["alert_id", "domain_id"])
    def create_alert_notification(self, params):
        """Create alert notification

        Args:
            params (dict): {
                'job_id': 'str',
                'alert_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """

        job_id = params.get("job_id")
        alert_id = params["alert_id"]
        domain_id = params["domain_id"]

        job_mgr: JobManager = self.locator.get_manager("JobManager")

        try:
            alert_mgr: AlertManager = self.locator.get_manager("AlertManager")

            alert_vo: Alert = alert_mgr.get_alert(alert_id, domain_id)
            project_id = alert_vo.project_id
            escalation_policy_id = alert_vo.escalation_policy_id

            alert_options = self._get_project_alert_options(project_id, domain_id)
            (
                rules,
                finish_condition,
            ) = self._get_escalation_policy_rules_and_finish_condition(
                escalation_policy_id, domain_id
            )

            # Check Notification Urgency and Finish Condition
            if not (
                self._check_notification_options(
                    alert_vo.urgency, alert_id, alert_options
                )
                and self._check_finish_condition(
                    alert_vo.state, alert_id, finish_condition
                )
            ):
                alert_mgr.update_alert_by_vo({"escalation_ttl": 0}, alert_vo)

            else:
                # Escalate Alert
                is_notify, alert_vo = self._check_escalation_time_and_escalate_alert(
                    alert_mgr, alert_vo, rules
                )
                if is_notify:
                    title = f"[Alerting] {alert_vo.title}"
                    current_rule = rules[alert_vo.escalation_step - 1]
                    notification_level = current_rule["notification_level"]

                    notification_mgr: NotificationManager = self.locator.get_manager(
                        "NotificationManager"
                    )
                    message = self._create_message(
                        alert_vo,
                        title,
                        "ERROR",
                        notification_level=notification_level,
                        has_callback=True,
                        has_short_description=True,
                    )
                    notification_mgr.create_notification(message)

            if job_id:
                job_vo = self.job_mgr.get_job(job_id, domain_id)
                job_mgr.decrease_remained_tasks(job_vo)
        except Exception as e:
            if job_id:
                job_vo = self.job_mgr.get_job(job_id, domain_id)
                job_mgr.change_error_status(job_vo, e)

            _LOGGER.error(f"[create_notification] Job Error: {e}", exc_info=True)
            self.transaction.execute_rollback()

    def _list_domains_of_alerts(self):
        alert_mgr: AlertManager = self.locator.get_manager("AlertManager")
        query = {
            "distinct": "domain_id",
            "filter": [
                {"k": "state", "v": ["TRIGGERED", "ACKNOWLEDGED"], "o": "in"},
                {"k": "escalation_ttl", "v": 0, "o": "gt"},
            ],
        }

        response = alert_mgr.stat_alerts(query)
        return response.get("results", [])

    def _list_open_alerts(self, domain_id):
        alert_mgr: AlertManager = self.locator.get_manager("AlertManager")
        project_alert_config_mgr: ProjectAlertConfigManager = self.locator.get_manager(
            "ProjectAlertConfigManager"
        )

        project_ids = []
        (
            project_alert_config_vos,
            total_count,
        ) = project_alert_config_mgr.list_project_alert_configs(
            {
                "filter": [{"k": "domain_id", "v": domain_id, "o": "eq"}],
                "only": ["project_id"],
            }
        )

        for project_alert_config_vo in project_alert_config_vos:
            project_ids.append(project_alert_config_vo.project_id)

        query = {
            "filter": [
                {"k": "domain_id", "v": domain_id, "o": "eq"},
                {"k": "project_id", "v": project_ids, "o": "in"},
                {"k": "state", "v": ["TRIGGERED", "ACKNOWLEDGED"], "o": "in"},
                {"k": "escalation_ttl", "v": 0, "o": "gt"},
            ]
        }

        return alert_mgr.list_alerts(query)

    @cache.cacheable(key="project-alert-options:{domain_id}:{project_id}", expire=300)
    def _get_project_alert_options(self, project_id, domain_id):
        project_alert_config_mgr: ProjectAlertConfigManager = self.locator.get_manager(
            "ProjectAlertConfigManager"
        )
        project_alert_config_vo: ProjectAlertConfig = (
            project_alert_config_mgr.get_project_alert_config(project_id, domain_id)
        )

        return dict(project_alert_config_vo.options.to_dict())

    @cache.cacheable(
        key="escalation-policy-condition:{domain_id}:{escalation_policy_id}", expire=300
    )
    def _get_escalation_policy_rules_and_finish_condition(
        self, escalation_policy_id, domain_id
    ):
        escalation_policy_mgr: EscalationPolicyManager = self.locator.get_manager(
            "EscalationPolicyManager"
        )
        escalation_policy_vo: EscalationPolicy = (
            escalation_policy_mgr.get_escalation_policy(escalation_policy_id, domain_id)
        )
        rules = []
        for rule in escalation_policy_vo.rules:
            rules.append(dict(rule.to_dict()))

        return rules, escalation_policy_vo.finish_condition

    @staticmethod
    def _get_current_escalation_rule(alert_vo: Alert, rules):
        return rules[alert_vo.escalation_step - 1]

    @staticmethod
    def _check_notification_options(alert_urgency, alert_id, alert_options):
        if (
            alert_options["notification_urgency"] == "HIGH_ONLY"
            and alert_urgency == "LOW"
        ):
            _LOGGER.debug(
                f"[_check_notification_options] Stop notifications. "
                f"(notification_urgency = HIGH_ONLY, alert_urgency = LOW, alert_id = {alert_id})"
            )
            return False
        else:
            return True

    @staticmethod
    def _check_finish_condition(alert_state, alert_id, finish_condition):
        if finish_condition == "ACKNOWLEDGED" and alert_state == "ACKNOWLEDGED":
            _LOGGER.debug(
                f"[_check_finish_condition] Stop notifications. "
                f"(finish_condition = ACKNOWLEDGED, alert_state = ACKNOWLEDGED, "
                f"alert_id = {alert_id})"
            )
            return False
        else:
            return True

    @staticmethod
    def _check_escalation_time_and_escalate_alert(
        alert_mgr: AlertManager, alert_vo: Alert, rules
    ):
        current_step = alert_vo.escalation_step
        escalation_ttl = alert_vo.escalation_ttl
        current_rule = rules[current_step - 1]
        escalate_minutes = current_rule.get("escalate_minutes", 0)
        escalated_at: Union[datetime, None] = alert_vo.escalated_at

        # First triggered alert
        if escalated_at is None:
            escalated_alert_vo = alert_mgr.update_alert_by_vo(
                {"escalated_at": datetime.utcnow()}, alert_vo
            )
            return True, escalated_alert_vo
        else:
            now = datetime.utcnow()

            # now > escalated_at + escalate_minutes
            if now > (escalated_at + timedelta(minutes=escalate_minutes)):
                # When the current step is the maximum
                if len(rules) == current_step:
                    if escalation_ttl == 1:
                        _LOGGER.debug(
                            f"[_check_escalation_time_and_escalate_alert] Max escalation step. "
                            f"(alert_id = {alert_vo.alert_id})"
                        )

                        escalated_alert_vo = alert_mgr.update_alert_by_vo(
                            {
                                "escalated_at": datetime.utcnow(),
                                "escalation_ttl": escalation_ttl - 1,
                            },
                            alert_vo,
                        )

                        return False, escalated_alert_vo
                    else:
                        _LOGGER.debug(
                            f"[_check_escalation_time_and_escalate_alert] Repeat again from the first step. "
                            f"(alert_id = {alert_vo.alert_id})"
                        )

                        escalated_alert_vo = alert_mgr.update_alert_by_vo(
                            {
                                "escalated_at": datetime.utcnow(),
                                "escalation_step": 1,
                                "escalation_ttl": escalation_ttl - 1,
                            },
                            alert_vo,
                        )

                        return True, escalated_alert_vo
                else:
                    _LOGGER.debug(
                        f"[_check_escalation_time_and_escalate_alert] Escalate from {current_step} "
                        f"to {current_step + 1} steps. (alert_id = {alert_vo.alert_id})"
                    )

                    escalated_alert_vo = alert_mgr.update_alert_by_vo(
                        {
                            "escalated_at": datetime.utcnow(),
                            "escalation_step": current_step + 1,
                        },
                        alert_vo,
                    )

                    return True, escalated_alert_vo
            else:
                return False, alert_vo

    def _create_message(
        self,
        alert_vo: Alert,
        title: str,
        notification_type: str,
        notification_level="ALL",
        has_callback=False,
        has_short_description=False,
        user_id=None,
    ):
        domain_id = alert_vo.domain_id
        project_name = self._get_project_name(alert_vo.project_id, domain_id)

        tags = [
            {
                "key": "Alert Number",
                "value": f"#{alert_vo.alert_number}",
                "options": {"short": True},
            },
            {"key": "State", "value": alert_vo.state, "options": {"short": True}},
            {"key": "Urgency", "value": alert_vo.urgency, "options": {"short": True}},
            {
                "key": "Triggered by",
                "value": self._get_triggered_by_name(alert_vo.triggered_by, domain_id),
                "options": {"short": True},
            },
            {"key": "Project", "value": project_name},
        ]

        resource = alert_vo.resource or {}

        if "name" in resource:
            tags.append({"key": "Resource", "value": resource["name"]})

        if alert_vo.assignee:
            tags.append(
                {
                    "key": "Assigned to",
                    "value": self._get_user_name(alert_vo.assignee, domain_id),
                }
            )

        if alert_vo.account:
            tags.append({"key": "Account", "value": alert_vo.account})

        callbacks = []

        if has_callback:
            access_key = self._generate_access_key()
            callback_url = self._make_callback_url(
                alert_vo.alert_id, domain_id, access_key
            )
            callbacks.append({"label": "Acknowledge Alerts", "url": callback_url})

        if user_id:
            resource_type = "identity.User"
            resource_id = user_id
        else:
            resource_type = "identity.Project"
            resource_id = alert_vo.project_id

        message = {
            "title": title,
            "tags": tags,
            "callbacks": callbacks,
            "occurred_at": utils.datetime_to_iso8601(alert_vo.created_at),
        }

        if alert_vo.description and alert_vo.description != "":
            message["description"] = alert_vo.description

        alert_link = self._make_alert_link(alert_vo.alert_id, alert_vo.domain_id)

        if alert_link:
            message["link"] = alert_link

        if alert_vo.image_url:
            message["image_url"] = alert_vo.image_url

        if has_short_description:
            # TODO: Need to change multiple language
            message[
                "short_description"
            ] = f'경고! 장애 발생! {project_name.replace(" >", "의")} 프로젝트에 장애가 발생했습니다.'

        return {
            "resource_type": resource_type,
            "resource_id": resource_id,
            "notification_type": notification_type,
            "topic": "monitoring.Alert",
            "message": message,
            "notification_level": notification_level,
            "domain_id": domain_id,
        }

    @cache.cacheable(key="project-name:{domain_id}:{project_id}", expire=300)
    def _get_project_name(self, project_id, domain_id):
        try:
            identity_mgr: IdentityManager = self.locator.get_manager("IdentityManager")
            project_info = identity_mgr.get_project(project_id, domain_id)
            return (
                f'{project_info["project_group_info"]["name"]} > {project_info["name"]}'
            )
        except Exception as e:
            _LOGGER.error(
                f"[_get_project_name] Failed to get project: {e}", exc_info=True
            )

        return project_id

    @cache.cacheable(key="triggered-by-name:{domain_id}:{triggered_by}", expire=300)
    def _get_triggered_by_name(self, triggered_by, domain_id):
        if triggered_by.startswith("webhook-"):
            try:
                webhook_mgr: WebhookManager = self.locator.get_manager("WebhookManager")
                webhook_info = webhook_mgr.get_webhook(triggered_by, domain_id)

                return webhook_info["name"]
            except Exception as e:
                _LOGGER.error(
                    f"[_get_triggered_by_name] Failed to get webhook: {e}",
                    exc_info=True,
                )
        else:
            return self._get_user_name(triggered_by, domain_id)

        return triggered_by

    @cache.cacheable(key="user-name:{domain_id}:{user_id}", expire=300)
    def _get_user_name(self, user_id, domain_id):
        try:
            identity_mgr: IdentityManager = self.locator.get_manager("IdentityManager")
            user_info = identity_mgr.get_user(user_id, domain_id)

            if len(user_info.get("name", "").strip()) == 0:
                return user_info["user_id"]
            else:
                return f'{user_info["user_id"]} ({user_info["name"]})'
        except Exception as e:
            _LOGGER.error(f"[_get_user_name] Failed to get user: {e}", exc_info=True)

        return user_id

    @staticmethod
    def _generate_access_key():
        return utils.random_string(16)

    def _make_callback_url(self, alert_id, domain_id, access_key):
        def _rollback(alert_id, access_key):
            _LOGGER.info(
                f"[_make_callback_url._rollback] "
                f"Delete cache : {alert_id} "
                f"({access_key})"
            )
            cache.delete(f"alert-notification-callback:{alert_id}:{access_key}")

        cache.set(
            f"alert-notification-callback:{alert_id}:{access_key}",
            domain_id,
            expire=600,
        )
        self.transaction.add_rollback(_rollback, alert_id, access_key)

        webhook_domain = config.get_global("WEBHOOK_DOMAIN")
        return (
            f"{webhook_domain}/monitoring/v1/alert/{alert_id}/{access_key}/ACKNOWLEDGED"
        )

    def _make_alert_link(self, alert_id, domain_id):
        console_domain: str = config.get_global("CONSOLE_DOMAIN")

        if console_domain.strip() != "":
            domain_name = self._get_domain_name(domain_id)
            console_domain = console_domain.format(domain_name=domain_name)

            return f"{console_domain}/alert-manager/alert/{alert_id}"

    @cache.cacheable(key="domain-name:{domain_id}", expire=3600)
    def _get_domain_name(self, domain_id):
        try:
            identity_mgr: IdentityManager = self.locator.get_manager("IdentityManager")
            domain_info = identity_mgr.get_domain(domain_id)
            return domain_info["name"]
        except Exception as e:
            _LOGGER.error(
                f"[_get_domain_name] Failed to get domain: {e}", exc_info=True
            )

        return domain_id



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class AlertResource(EmbeddedDocument):
    resource_id = StringField(default=None, null=True)
    resource_type = StringField(default=None, null=True)
    name = StringField(default=None, null=True)


class AlertNumber(MongoModel):
    next = IntField(default=1)
    domain_id = StringField(max_length=40)


class Alert(MongoModel):
    alert_number = IntField(required=True)
    alert_number_str = StringField()
    alert_id = StringField(max_length=40, generate_id="alert", unique=True)
    title = StringField()
    state = StringField(
        max_length=20,
        default="TRIGGERED",
        choices=("TRIGGERED", "ACKNOWLEDGED", "RESOLVED", "ERROR"),
    )
    description = StringField(default=None, null=True)
    assignee = StringField(default=None, null=True)
    urgency = StringField(max_length=20, default="HIGH", choices=("HIGH", "LOW"))
    severity = StringField(
        max_length=20,
        default="NONE",
        choices=("CRITICAL", "ERROR", "WARNING", "INFO", "NOT_AVAILABLE", "NONE"),
    )
    rule = StringField(default=None, null=True)
    image_url = StringField(default=None, null=True)
    resource = EmbeddedDocumentField(AlertResource, default=None, null=True)
    provider = StringField(default=None, null=True)
    account = StringField(default=None, null=True)
    additional_info = DictField()
    escalation_step = IntField(default=1)
    escalation_ttl = IntField(default=0)
    triggered_by = StringField(default=None, null=True)
    webhook_id = StringField(max_length=40, default=None, null=True)
    escalation_policy_id = StringField(max_length=40)
    project_id = StringField(max_length=40)
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    updated_at = DateTimeField(auto_now=True)
    acknowledged_at = DateTimeField(default=None, null=True)
    resolved_at = DateTimeField(default=None, null=True)
    escalated_at = DateTimeField(default=None, null=True)

    meta = {
        "updatable_fields": [
            "title",
            "state",
            "description",
            "assignee",
            "urgency",
            "escalation_step",
            "escalation_ttl",
            "escalation_policy_id",
            "project_id",
            "acknowledged_at",
            "resolved_at",
            "escalated_at",
        ],
        "minimal_fields": [
            "alert_number",
            "alert_id",
            "title",
            "state",
            "assignee",
            "urgency",
            "escalation_step",
            "escalation_ttl",
            "project_id",
        ],
        "change_query_keys": {
            "resource_id": "resource.resource_id",
            "user_projects": "project_id",
        },
        "ordering": ["-created_at"],
        "indexes": [
            "alert_number",
            # 'alert_id',
            "state",
            "assignee",
            "urgency",
            "severity",
            "resource.resource_id",
            "resource.resource_type",
            "resource.name",
            "provider",
            "account",
            "escalation_step",
            "triggered_by",
            "webhook_id",
            "escalation_policy_id",
            "project_id",
            "workspace_id",
            "domain_id",
            "created_at",
            "acknowledged_at",
            "resolved_at",
            "escalated_at",
            {
                "fields": [
                    "domain_id",
                    "workspace_id",
                    "state",
                    "escalation_step",
                    "escalation_ttl",
                    "escalation_policy_id",
                    "escalated_at",
                ],
                "name": "COMPOUND_INDEX_FOR_ESCALATION",
            },
        ],
    }



import logging

from spaceone.core import utils
from spaceone.core.service import *

from spaceone.monitoring.error import *
from spaceone.monitoring.manager.project_alert_config_manager import (
    ProjectAlertConfigManager,
)
from spaceone.monitoring.manager.repository_manager import RepositoryManager
from spaceone.monitoring.manager.webhook_manager import WebhookManager
from spaceone.monitoring.manager.webhook_plugin_manager import WebhookPluginManager
from spaceone.monitoring.model.webhook_model import Webhook

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class WebhookService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.webhook_mgr: WebhookManager = self.locator.get_manager("WebhookManager")
        self.webhook_plugin_mgr: WebhookPluginManager = self.locator.get_manager(
            "WebhookPluginManager"
        )

    @transaction(
        permission="monitoring:Webhook.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["name", "plugin_info", "project_id", "domain_id", "workspace_id"])
    def create(self, params: dict) -> Webhook:
        """Create webhook

        Args:
            params (dict): {
                'name': 'str',
                'plugin_info': 'dict',
                'project_id': 'str'
                'tags': 'dict',
                'domain_id': 'str',          # injected from auth (required)
                'workspace_id': 'str',       # injected from auth (required)
            }

        Returns:
            webhook_vo (object)
        """

        project_id = params["project_id"]
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]

        project_alert_config_mgr: ProjectAlertConfigManager = self.locator.get_manager(
            "ProjectAlertConfigManager"
        )

        project_alert_config_mgr.get_project_alert_config(
            project_id, domain_id, workspace_id
        )

        self._check_plugin_info(params["plugin_info"])
        plugin_info = self._get_plugin(params["plugin_info"], domain_id)
        params["capability"] = plugin_info.get("capability", {})

        _LOGGER.debug(f'[create] Init Plugin: {params["plugin_info"]}')
        endpoint, updated_version = self.webhook_plugin_mgr.get_webhook_plugin_endpoint(
            params["plugin_info"], domain_id
        )
        if updated_version:
            params["plugin_info"]["version"] = updated_version

        options = params["plugin_info"].get("options", {})
        plugin_metadata = self._init_plugin(endpoint, options)
        params["plugin_info"]["metadata"] = plugin_metadata

        webhook_vo: Webhook = self.webhook_mgr.create_webhook(params)

        access_key = self._generate_access_key()
        webhook_url = self._make_webhook_url(webhook_vo.webhook_id, access_key)

        return self.webhook_mgr.update_webhook_by_vo(
            {"access_key": access_key, "webhook_url": webhook_url}, webhook_vo
        )

    @transaction(
        permission="monitoring:Webhook.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["webhook_id", "domain_id", "workspace_id"])
    def update(self, params: dict) -> Webhook:
        """Update webhook

        Args:
            params (dict): {
                'webhook_id': 'str',      # required
                'name': 'dict',
                'tags': 'dict'
                'domain_id': 'str',       # injected from auth (required)
                'workspace_id': 'str',    # injected from auth (required)
                'user_projects': 'list',  # injected from auth
            }

        Returns:
            webhook_vo (object)
        """

        webhook_id = params["webhook_id"]
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]
        user_projects = params.get("user_projects")

        webhook_vo = self.webhook_mgr.get_webhook(
            webhook_id, domain_id, workspace_id, user_projects
        )

        return self.webhook_mgr.update_webhook_by_vo(params, webhook_vo)

    @transaction(
        permission="monitoring:Webhook.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["webhook_id", "domain_id", "workspace_id"])
    def enable(self, params: dict) -> Webhook:
        """Enable webhook

        Args:
            params (dict): {
                'webhook_id': 'str',      # required
                'domain_id': 'str',       # injected from auth (required)
                'workspace_id': 'str',    # injected from auth (required)
                'user_projects': 'list',  # injected from auth
            }

        Returns:
            webhook_vo (object)
        """

        webhook_id = params["webhook_id"]
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]
        user_projects = params.get("user_projects")

        webhook_vo = self.webhook_mgr.get_webhook(
            webhook_id, domain_id, workspace_id, user_projects
        )

        return self.webhook_mgr.update_webhook_by_vo({"state": "ENABLED"}, webhook_vo)

    @transaction(
        permission="monitoring:Webhook.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["webhook_id", "domain_id", "workspace_id"])
    def disable(self, params: dict) -> Webhook:
        """Disable webhook

        Args:
            params (dict): {
                'webhook_id': 'str',      # required
                'domain_id': 'str',       # injected from auth (required)
                'workspace_id': 'str',    # injected from auth (required)
                'user_projects': 'list',  # injected from auth
            }

        Returns:
            webhook_vo (object)
        """

        webhook_id = params["webhook_id"]
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]
        user_projects = params.get("user_projects")

        webhook_vo = self.webhook_mgr.get_webhook(
            webhook_id, domain_id, workspace_id, user_projects
        )

        return self.webhook_mgr.update_webhook_by_vo({"state": "DISABLED"}, webhook_vo)

    @transaction(
        permission="monitoring:Webhook.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["webhook_id", "domain_id", "workspace_id"])
    def delete(self, params: dict) -> None:
        """Delete webhook

        Args:
            params (dict): {
                'webhook_id': 'str',      # required
                'domain_id': 'str',       # injected from auth (required)
                'workspace_id': 'str',    # injected from auth (required)
                'user_projects': 'list',  # injected from auth
            }

        Returns:
            None
        """

        self.webhook_mgr.delete_webhook(
            params["webhook_id"],
            params["domain_id"],
            params["workspace_id"],
            params.get("user_projects"),
        )

    @transaction(
        permission="monitoring:Webhook.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["webhook_id", "domain_id"])
    def verify_plugin(self, params):
        """Verify webhook plugin

        Args:
            params (dict): {
                'webhook_id': 'str',      # required
                'domain_id': 'str',       # injected from auth (required)
                'workspace_id': 'str',    # injected from auth (required)
                'user_projects': 'list',  # injected from auth
            }

        Returns:
            webhook_vo (object)
        """

        webhook_id = params["webhook_id"]
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]
        user_projects = params.get("user_projects")

        webhook_vo = self.webhook_mgr.get_webhook(
            webhook_id, domain_id, workspace_id, user_projects
        )

        endpoint = self.webhook_plugin_mgr.get_webhook_plugin_endpoint_by_vo(webhook_vo)

        self._verify_plugin(endpoint, webhook_vo.plugin_info.options)

    @transaction(
        permission="monitoring:Webhook.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["webhook_id", "domain_id", "workspace_id"])
    def update_plugin(self, params):
        """Update webhook plugin

        Args:
            params (dict): {
                'webhook_id': 'str',      # required
                'version': 'str',
                'options': 'dict',
                'upgrade_mode': 'str',
                'domain_id': 'str',       # injected from auth (required)
                'workspace_id': 'str',    # injected from auth (required)
                'user_projects': 'list',  # injected from auth
            }

        Returns:
            webhook_vo (object)
        """

        webhook_id = params["webhook_id"]
        options = params.get("options")
        version = params.get("version")
        upgrade_mode = params.get("upgrade_mode")

        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]
        user_projects = params.get("user_projects")

        webhook_vo = self.webhook_mgr.get_webhook(
            webhook_id, domain_id, workspace_id, user_projects
        )
        plugin_info = webhook_vo.plugin_info.to_dict()

        if version:
            plugin_info["version"] = version

        if options:
            plugin_info["options"] = options

        if upgrade_mode:
            plugin_info["upgrade_mode"] = upgrade_mode

        endpoint, updated_version = self.webhook_plugin_mgr.get_webhook_plugin_endpoint(
            plugin_info, domain_id
        )
        if updated_version:
            plugin_info["version"] = updated_version

        plugin_metadata = self._init_plugin(endpoint, plugin_info.get("options", {}))
        plugin_info["metadata"] = plugin_metadata

        params = {"plugin_info": plugin_info}

        _LOGGER.debug(f"[update_plugin] {plugin_info}")

        return self.webhook_mgr.update_webhook_by_vo(params, webhook_vo)

    @transaction(
        permission="monitoring:Webhook.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["webhook_id", "domain_id", "workspace_id"])
    def get(self, params):
        """Get webhook

        Args:
            params (dict): {
                'webhook_id': 'str',      # required
                'domain_id': 'str',       # injected from auth (required)
                'workspace_id': 'str',    # injected from auth (required)
                'user_projects': 'list',  # injected from auth
            }

        Returns:
            webhook_vo (object)
        """

        return self.webhook_mgr.get_webhook(
            params["webhook_id"],
            params["domain_id"],
            params["workspace_id"],
            params.get("user_projects"),
        )

    @transaction(
        permission="monitoring:Webhook.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["domain_id"])
    @append_query_filter(
        [
            "webhook_id",
            "name",
            "state",
            "access_key",
            "project_id",
            "domain_id",
            "workspace_id",
            "user_projects",
        ]
    )
    @append_keyword_filter(["webhook_id", "name"])
    def list(self, params):
        """List webhooks

        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'webhook_id': 'str',
                'name': 'str',
                'state': 'str',
                'project_id': 'str',
                'domain_id': 'str',       # injected from auth (required)
                'user_projects': 'list',  # injected from auth
            }

        Returns:
            webhook_vos (object)
            total_count
        """

        query = params.get("query", {})
        return self.webhook_mgr.list_webhooks(query)

    @transaction(
        permission="monitoring:Webhook.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["query", "domain_id", "workspace_id"])
    @append_query_filter(["domain_id", "workspace_id", "user_projects"])
    @append_keyword_filter(["webhook_id", "name"])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'domain_id': 'str',          # injected from auth (required)
                'workspace_id': 'str',       # injected from auth (required)
                'user_projects': 'list',     # injected from auth
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get("query", {})
        return self.webhook_mgr.stat_webhooks(query)

    @staticmethod
    def _generate_access_key() -> str:
        return utils.random_string(16)

    @staticmethod
    def _make_webhook_url(webhook_id: str, access_key: str):
        return f"/monitoring/v1/webhook/{webhook_id}/{access_key}/events"

    @staticmethod
    def _check_plugin_info(plugin_info_params: dict) -> None:
        if "plugin_id" not in plugin_info_params:
            raise ERROR_REQUIRED_PARAMETER(key="plugin_info.plugin_id")

    def _get_plugin(self, plugin_info: dict, domain_id: str):
        plugin_id = plugin_info["plugin_id"]

        repo_mgr: RepositoryManager = self.locator.get_manager("RepositoryManager")
        plugin_info = repo_mgr.get_plugin(plugin_id, domain_id)

        return plugin_info

    def _init_plugin(self, endpoint: str, options: dict):
        self.webhook_plugin_mgr.initialize(endpoint)
        return self.webhook_plugin_mgr.init_plugin(options)

    def _verify_plugin(self, endpoint: str, options: dict):
        self.webhook_plugin_mgr.initialize(endpoint)
        self.webhook_plugin_mgr.verify_plugin(options)



import logging

from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.monitoring.error import *

_LOGGER = logging.getLogger(__name__)


class InventoryManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.inventory_connector: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", service="inventory"
        )

    def get_server(self, server_id, domain_id):
        return self.inventory_connector.dispatch(
            "Server.get", {"server_id": server_id, "domain_id": domain_id}
        )

    def list_servers(self, query, domain_id):
        return self.inventory_connector.dispatch(
            "Server.list", {"query": query, "domain_id": domain_id}
        )

    def get_cloud_service(self, cloud_service_id, domain_id):
        return self.inventory_connector.dispatch(
            "CloudService.get",
            {"cloud_service_id": cloud_service_id, "domain_id": domain_id},
        )

    def list_cloud_services(self, query, domain_id):
        return self.inventory_connector.dispatch(
            "CloudService.list", {"query": query, "domain_id": domain_id}
        )

    def get_resource(self, resource_type, resource_id, domain_id):
        return self.get_cloud_service(resource_id, domain_id)

    def list_resources(self, resources, required_keys, domain_id):
        query = self._make_query(resources, required_keys)
        response = self.list_cloud_services(query, domain_id)
        return response.get("results", [])

    @staticmethod
    def _make_query(resources, required_keys):
        only_keys = list(
            set(
                ["cloud_service_id", "collection_info.secret_id", "region_code"]
                + required_keys
            )
        )
        only_keys.sort()

        return {
            "filter": [{"k": "cloud_service_id", "v": resources, "o": "in"}],
            "only": only_keys,
        }



import functools

from spaceone.api.monitoring.v1 import note_pb2
from spaceone.core import utils

from spaceone.monitoring.model.note_model import Note

__all__ = ["NoteInfo", "NotesInfo"]


def NoteInfo(note_vo: Note, minimal=False):
    info = {
        "note_id": note_vo.note_id,
        "note": note_vo.note,
        "alert_id": note_vo.alert_id,
        "created_by": note_vo.created_by,
        "workspace_id": note_vo.workspace_id,
        "project_id": note_vo.project_id,
    }

    if not minimal:
        info.update(
            {
                "domain_id": note_vo.domain_id,
                "created_at": utils.datetime_to_iso8601(note_vo.created_at),
            }
        )

    return note_pb2.NoteInfo(**info)


def NotesInfo(note_vos, total_count, **kwargs):
    return note_pb2.NotesInfo(
        results=list(map(functools.partial(NoteInfo, **kwargs), note_vos)),
        total_count=total_count,
    )



import logging

from spaceone.core import cache
from spaceone.core import config
from spaceone.core.service import *

from spaceone.monitoring.error import *
from spaceone.monitoring.manager.data_source_manager import DataSourceManager
from spaceone.monitoring.manager.data_source_plugin_manager import (
    DataSourcePluginManager,
)
from spaceone.monitoring.manager.repository_manager import RepositoryManager
from spaceone.monitoring.manager.secret_manager import SecretManager
from spaceone.monitoring.model import DataSource

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class DataSourceService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.data_source_mgr: DataSourceManager = self.locator.get_manager(
            "DataSourceManager"
        )
        self.ds_plugin_mgr: DataSourcePluginManager = self.locator.get_manager(
            "DataSourcePluginManager"
        )

    @transaction(permission="monitoring:DataSource.write", role_types=["DOMAIN_ADMIN"])
    @check_required(["name", "plugin_info", "domain_id"])
    def register(self, params: dict) -> DataSource:
        """Register data source

        Args:
            params (dict): {
                'name': 'str',           # required
                'plugin_info': 'dict',   # required
                'tags': 'dict',
                'domain_id': 'str'       # injected from auth (required)
            }

        Returns:
            data_source_vo (object)
        """

        domain_id = params["domain_id"]

        self._check_plugin_info(params["plugin_info"])
        plugin_info = self._get_plugin(params["plugin_info"], domain_id)
        params["capability"] = plugin_info.get("capability", {})
        params["provider"] = plugin_info.get("provider")

        self._check_plugin_capability(params["capability"])
        params["monitoring_type"] = params["capability"]["monitoring_type"]

        # Update metadata
        endpoint, updated_version = self.ds_plugin_mgr.get_data_source_plugin_endpoint(
            params["plugin_info"], domain_id
        )
        if updated_version:
            params["plugin_info"]["version"] = updated_version

        options = params["plugin_info"].get("options", {})
        plugin_metadata = self._init_plugin(
            endpoint, options, params["monitoring_type"]
        )
        params["plugin_info"]["metadata"] = plugin_metadata

        return self.data_source_mgr.register_data_source(params)

    @transaction(permission="monitoring:DataSource.write", role_types=["DOMAIN_ADMIN"])
    @check_required(["data_source_id", "domain_id"])
    def update(self, params):
        """Update data source

        Args:
            params (dict): {
                'data_source_id': 'str',   # required
                'name': 'dict',
                'tags': 'dict'
                'domain_id': 'str'         # injected from auth (required)
            }

        Returns:
            data_source_vo (object)
        """
        data_source_id = params["data_source_id"]
        domain_id = params["domain_id"]
        data_source_vo = self.data_source_mgr.get_data_source(data_source_id, domain_id)

        return self.data_source_mgr.update_data_source_by_vo(params, data_source_vo)

    @transaction(permission="monitoring:DataSource.write", role_types=["DOMAIN_ADMIN"])
    @check_required(["data_source_id", "domain_id"])
    def enable(self, params: dict) -> DataSource:
        """Enable data source

        Args:
            params (dict): {
                'data_source_id': 'str',   # required
                'domain_id': 'str'         # injected from auth (required)
            }

        Returns:
            data_source_vo (object)
        """

        data_source_id = params["data_source_id"]
        domain_id = params["domain_id"]
        data_source_vo = self.data_source_mgr.get_data_source(data_source_id, domain_id)

        return self.data_source_mgr.update_data_source_by_vo(
            {"state": "ENABLED"}, data_source_vo
        )

    @transaction(permission="monitoring:DataSource.write", role_types=["DOMAIN_ADMIN"])
    @check_required(["data_source_id", "domain_id"])
    def disable(self, params: dict) -> DataSource:
        """Disable data source

        Args:
            params (dict): {
                'data_source_id': 'str',   # required
                'domain_id': 'str'         # injected from auth (required)
            }

        Returns:
            data_source_vo (object)
        """

        data_source_id = params["data_source_id"]
        domain_id = params["domain_id"]
        data_source_vo = self.data_source_mgr.get_data_source(data_source_id, domain_id)

        return self.data_source_mgr.update_data_source_by_vo(
            {"state": "DISABLED"}, data_source_vo
        )

    @transaction(permission="monitoring:DataSource.write", role_types=["DOMAIN_ADMIN"])
    @check_required(["data_source_id", "domain_id"])
    def deregister(self, params: dict) -> None:
        """Deregister data source

        Args:
            params (dict): {
                'data_source_id': 'str',   # required
                'domain_id': 'str'         # injected from auth (required)
            }

        Returns:
            None
        """

        self.data_source_mgr.deregister_data_source(
            params["data_source_id"], params["domain_id"]
        )

    @transaction(permission="monitoring:DataSource.write", role_types=["DOMAIN_ADMIN"])
    @check_required(["data_source_id", "domain_id"])
    def verify_plugin(self, params: dict) -> None:
        """Verify data source plugin

        Args:
            params (dict): {
                'data_source_id': 'str',   # required
                'domain_id': 'str'         # injected from auth (required)
            }

        Returns:
            data_source_vo (object)
        """

        data_source_id = params["data_source_id"]
        domain_id = params["domain_id"]
        data_source_vo = self.data_source_mgr.get_data_source(data_source_id, domain_id)

        endpoint = self.ds_plugin_mgr.get_data_source_plugin_endpoint_by_vo(
            data_source_vo
        )

        self._verify_plugin(
            endpoint,
            data_source_vo.plugin_info.to_dict(),
            data_source_vo.capability,
            domain_id,
        )

    @transaction(permission="monitoring:DataSource.write", role_types=["DOMAIN_ADMIN"])
    @check_required(["data_source_id", "domain_id"])
    def update_plugin(self, params):
        """Update data source plugin

        Args:
            params (dict): {
                'data_source_id': 'str',   # required
                'version': 'str',
                'options': 'dict',
                'upgrade_mode': 'str',
                'domain_id': 'str'         # injected from auth (required)
            }

        Returns:
            data_source_vo (object)
        """
        data_source_id = params["data_source_id"]
        domain_id = params["domain_id"]
        version = params.get("version")
        options = params.get("options")
        upgrade_mode = params.get("upgrade_mode")

        data_source_vo = self.data_source_mgr.get_data_source(data_source_id, domain_id)

        plugin_info = data_source_vo.plugin_info.to_dict()

        if version:
            plugin_info["version"] = version

        if options:
            plugin_info["options"] = options

        if upgrade_mode:
            plugin_info["upgrade_mode"] = upgrade_mode

        endpoint, updated_version = self.ds_plugin_mgr.get_data_source_plugin_endpoint(
            plugin_info, domain_id
        )
        if updated_version:
            plugin_info["version"] = updated_version

        plugin_metadata = self._init_plugin(
            endpoint, plugin_info.get("options", {}), data_source_vo.monitoring_type
        )
        plugin_info["metadata"] = plugin_metadata

        params = {"plugin_info": plugin_info}

        _LOGGER.debug(f"[update_plugin] {plugin_info}")

        return self.data_source_mgr.update_data_source_by_vo(params, data_source_vo)

    @transaction(
        permission="monitoring:DataSource.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["data_source_id", "domain_id"])
    def get(self, params):
        """Get data source

        Args:
            params (dict): {
                'data_source_id': 'str',    # required
                'domain_id': 'str',         # injected from auth (required)
            }

        Returns:
            data_source_vo (object)
        """
        domain_id = params["domain_id"]
        self._initialize_data_sources(domain_id)

        return self.data_source_mgr.get_data_source(params["data_source_id"], domain_id)

    @transaction(
        permission="monitoring:DataSource.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["domain_id"])
    @append_query_filter(
        ["data_source_id", "name", "state", "monitoring_type", "provider", "domain_id"]
    )
    @append_keyword_filter(["data_source_id", "name", "provider"])
    def list(self, params):
        """List data sources

        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)'
                'data_source_id': 'str',
                'name': 'str',
                'state': 'str',
                'monitoring_type': 'str',
                'provider': 'str',
                'domain_id': 'str',             # injected from auth (required)
            }

        Returns:
            data_source_vos (object)
            total_count
        """
        domain_id = params["domain_id"]
        query = params.get("query", {})
        self._initialize_data_sources(domain_id)

        return self.data_source_mgr.list_data_sources(query)

    @transaction(
        permission="monitoring:DataSource.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["query", "domain_id"])
    @append_query_filter(["domain_id"])
    @append_keyword_filter(["data_source_id", "name", "provider"])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)'
                'domain_id': 'str',        # injected from auth (required)
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get("query", {})
        return self.data_source_mgr.stat_data_sources(query)

    @staticmethod
    def _check_plugin_capability(capability: dict) -> None:
        if "monitoring_type" not in capability:
            raise ERROR_WRONG_PLUGIN_SETTINGS(key="capability.monitoring_type")
        else:
            if capability["monitoring_type"] not in ["METRIC", "LOG"]:
                raise ERROR_WRONG_PLUGIN_SETTINGS(key="capability.monitoring_type")

        if "supported_schema" not in capability:
            raise ERROR_WRONG_PLUGIN_SETTINGS(key="capability.supported_schema")

    @staticmethod
    def _check_plugin_info(plugin_info_params: dict) -> None:
        if "plugin_id" not in plugin_info_params:
            raise ERROR_REQUIRED_PARAMETER(key="plugin_info.plugin_id")

        if (
            plugin_info_params.get("upgrade_mode", "AUTO") == "MANUAL"
            and "version" not in plugin_info_params
        ):
            raise ERROR_REQUIRED_PARAMETER(key="plugin_info.version")

        secret_id = plugin_info_params.get("secret_id")
        provider = plugin_info_params.get("provider")

        if secret_id is None and provider is None:
            raise ERROR_REQUIRED_PARAMETER(key="plugin_info.[secret_id | provider]")

    def _get_plugin(self, plugin_info: dict, domain_id: str) -> dict:
        plugin_id = plugin_info["plugin_id"]

        repo_mgr: RepositoryManager = self.locator.get_manager("RepositoryManager")
        return repo_mgr.get_plugin(plugin_id, domain_id)

    def _init_plugin(self, endpoint: str, options: dict, monitoring_type: str) -> dict:
        self.ds_plugin_mgr.initialize(endpoint)
        return self.ds_plugin_mgr.init_plugin(options, monitoring_type)

    def _verify_plugin(
        self, endpoint: str, plugin_info: dict, capability: dict, domain_id: str
    ) -> None:
        options = plugin_info.get("options", {})
        secret_mgr: SecretManager = self.locator.get_manager("SecretManager")
        secret_data, schema_id = secret_mgr.get_secret_data_from_plugin(
            plugin_info, capability, domain_id
        )

        ds_plugin_mgr: DataSourcePluginManager = self.locator.get_manager(
            "DataSourcePluginManager"
        )
        ds_plugin_mgr.initialize(endpoint)
        ds_plugin_mgr.verify_plugin(options, secret_data, schema_id)

    @cache.cacheable(key="init-data-source:{domain_id}", expire=300)
    def _initialize_data_sources(self, domain_id):
        _LOGGER.debug(f"[_initialize_data_source] domain_id: {domain_id}")

        query = {"filter": [{"k": "domain_id", "v": domain_id, "o": "eq"}]}
        data_source_vos, total_count = self.data_source_mgr.list_data_sources(query)

        installed_data_sources_ids = [
            data_source_vo.plugin_info.plugin_id for data_source_vo in data_source_vos
        ]
        _LOGGER.debug(
            f"[_initialize_data_source] Installed Plugins : {installed_data_sources_ids}"
        )

        global_conf = config.get_global()
        for _data_source in global_conf.get("INSTALLED_DATA_SOURCE_PLUGINS", []):
            if (
                _data_source["plugin_info"]["plugin_id"]
                not in installed_data_sources_ids
            ):
                try:
                    _LOGGER.debug(
                        f'[_initialize_data_source] Create init data source: {_data_source["plugin_info"]["plugin_id"]}'
                    )
                    _data_source["domain_id"] = domain_id
                    self.register(_data_source)
                except Exception as e:
                    _LOGGER.error(f"[_initialize_data_source] {e}")

        return True



from typing import List, Literal, Union
from datetime import datetime
from pydantic import BaseModel
from spaceone.core import utils

__all__ = ["EventResponse", "EventsResponse"]

EventType = Literal["ALERT", "RECOVERY"]
Severity = Literal["CRITICAL", "ERROR", "WARNING", "INFO", "NOT_AVAILABLE", "NONE"]


class Resource(BaseModel):
    resource_id: Union[str, None] = None
    name: Union[str, None] = None
    resource_type: Union[str, None] = None


class EventResponse(BaseModel):
    event_key: str
    event_type: EventType
    title: str
    description: Union[str, None] = None
    severity: Severity
    resource: Resource = {}
    rule: Union[str, None] = None
    occurred_at: datetime
    additional_info: dict
    image_url: Union[str, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["occurred_at"] = utils.datetime_to_iso8601(data["occurred_at"])
        return data


class EventsResponse(BaseModel):
    results: List[EventResponse]



from spaceone.core.pygrpc import BaseAPI
from spaceone.api.monitoring.plugin import webhook_pb2, webhook_pb2_grpc
from spaceone.monitoring.plugin.webhook.service.webhook_service import WebhookService


class Webhook(BaseAPI, webhook_pb2_grpc.WebhookServicer):
    pb2 = webhook_pb2
    pb2_grpc = webhook_pb2_grpc

    def init(self, request, context):
        params, metadata = self.parse_request(request, context)
        webhook_svc = WebhookService(metadata)
        response: dict = webhook_svc.init(params)
        return self.dict_to_message(response)

    def verify(self, request, context):
        params, metadata = self.parse_request(request, context)
        webhook_svc = WebhookService(metadata)
        webhook_svc.verify(params)
        return self.empty()



import logging

from spaceone.core.manager import BaseManager

from spaceone.monitoring.connector.datasource_plugin_connector import (
    DataSourcePluginConnector,
)
from spaceone.monitoring.error import *
from spaceone.monitoring.manager.plugin_manager import PluginManager
from spaceone.monitoring.model.data_source_model import DataSource
from spaceone.monitoring.model.plugin_metadata_model import (
    MetricPluginMetadataModel,
    LogPluginMetadataModel,
)

_LOGGER = logging.getLogger(__name__)


class DataSourcePluginManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.dsp_connector: DataSourcePluginConnector = self.locator.get_connector(
            "DataSourcePluginConnector"
        )

    def initialize(self, endpoint: str) -> None:
        _LOGGER.debug(f"[initialize] data source plugin endpoint: {endpoint}")
        self.dsp_connector.initialize(endpoint)

    def init_plugin(self, options: dict, monitoring_type: str) -> dict:
        plugin_info = self.dsp_connector.init(options)

        _LOGGER.debug(f"[plugin_info] {plugin_info}")
        plugin_metadata = plugin_info.get("metadata", {})

        self._validate_plugin_metadata(plugin_metadata, monitoring_type)

        return plugin_metadata

    def verify_plugin(self, options, secret_data, schema_id):
        self.dsp_connector.verify(options, secret_data, schema_id)

    def list_metrics(self, schema, options, secret_data, query):
        return self.dsp_connector.list_metrics(schema, options, secret_data, query)

    def get_metric_data(self, params):
        return self.dsp_connector.get_metric_data(params)

    def list_logs(
        self, schema, options, secret_data, query, keyword, start, end, sort, limit
    ):
        results = []

        for result in self.dsp_connector.list_logs(
            schema, options, secret_data, query, keyword, start, end, sort, limit
        ):
            results.extend(result.get("results", []))

        return {"results": results}

    @staticmethod
    def _validate_plugin_metadata(plugin_metadata: dict, monitoring_type: str) -> None:
        try:
            if monitoring_type == "METRIC":
                MetricPluginMetadataModel(plugin_metadata).validate()
            else:
                LogPluginMetadataModel(plugin_metadata).validate()

        except Exception as e:
            raise ERROR_INVALID_PLUGIN_OPTIONS(reason=str(e))

    @staticmethod
    def _process_stream(response_stream):
        for response in response_stream:
            yield response

    def get_data_source_plugin_endpoint_by_vo(self, data_source_vo: DataSource):
        plugin_info = data_source_vo.plugin_info.to_dict()
        endpoint, updated_version = self.get_data_source_plugin_endpoint(
            plugin_info, data_source_vo.domain_id
        )

        if updated_version:
            _LOGGER.debug(
                f'[get_data_source_plugin_endpoint_by_vo] upgrade plugin version: {plugin_info["version"]} -> {updated_version}'
            )
            self.upgrade_data_source_plugin_version(
                data_source_vo, endpoint, updated_version
            )

        return endpoint

    def get_data_source_plugin_endpoint(
        self, plugin_info: dict, domain_id: str
    ) -> dict:
        plugin_mgr: PluginManager = self.locator.get_manager("PluginManager")
        return plugin_mgr.get_plugin_endpoint(plugin_info, domain_id)

    def upgrade_data_source_plugin_version(
        self, data_source_vo: DataSource, endpoint, updated_version
    ):
        plugin_info = data_source_vo.plugin_info.to_dict()
        self.initialize(endpoint)
        plugin_metadata = self.init_plugin(
            plugin_info.get("options", {}), data_source_vo.monitoring_type
        )
        plugin_info["version"] = updated_version
        plugin_info["metadata"] = plugin_metadata
        data_source_vo.update({"plugin_info": plugin_info})



import logging

from spaceone.core.service import *

from spaceone.monitoring.error import *
from spaceone.monitoring.manager.escalation_policy_manager import (
    EscalationPolicyManager,
)
from spaceone.monitoring.manager.identity_manager import IdentityManager
from spaceone.monitoring.manager.project_alert_config_manager import (
    ProjectAlertConfigManager,
)
from spaceone.monitoring.model.project_alert_config_model import ProjectAlertConfig

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class ProjectAlertConfigService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.project_alert_config_mgr: ProjectAlertConfigManager = (
            self.locator.get_manager("ProjectAlertConfigManager")
        )

    @transaction(
        permission="monitoring:ProjectAlertConfig.write",
        role_types=["WORKSPACE_MEMBER"],
    )
    @check_required(["project_id", "domain_id", "workspace_id"])
    def create(self, params: dict) -> ProjectAlertConfig:
        """Create project alert configuration

        Args:
            params (dict): {
                'project_id': 'str',            # required
                'escalation_policy_id': 'str',
                'options': 'dict',
                'domain_id': 'str'              # injected from auth (required)
                'workspace_id': 'str',          # injected from auth (required)
            }

        Returns:
            project_alert_config_vo (object)
        """

        project_id = params["project_id"]
        escalation_policy_id = params.get("escalation_policy_id")
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]

        identity_mgr: IdentityManager = self.locator.get_manager("IdentityManager")
        escalation_policy_mgr: EscalationPolicyManager = self.locator.get_manager(
            "EscalationPolicyManager"
        )

        identity_mgr.get_project(project_id)

        if escalation_policy_id:
            escalation_policy_vo = escalation_policy_mgr.get_escalation_policy(
                escalation_policy_id, workspace_id, domain_id
            )
            if (
                escalation_policy_vo.resource_group == "PROJECT"
                and escalation_policy_vo.project_id != project_id
            ):
                raise ERROR_INVALID_ESCALATION_POLICY(
                    escalation_policy_id=escalation_policy_id
                )

            params["escalation_policy"] = escalation_policy_vo

        else:
            escalation_policy_vo = escalation_policy_mgr.get_default_escalation_policy(
                workspace_id, domain_id, project_id
            )
            params["escalation_policy_id"] = escalation_policy_vo.escalation_policy_id
            params["escalation_policy"] = escalation_policy_vo

        return self.project_alert_config_mgr.create_project_alert_config(params)

    @transaction(
        permission="monitoring:ProjectAlertConfig.write",
        role_types=["WORKSPACE_MEMBER"],
    )
    @check_required(["project_id", "domain_id", "workspace_id"])
    def update(self, params):
        """Update project alert configuration

        Args:
            params (dict): {
                'project_id': 'str',             # required
                'escalation_policy_id': 'dict',
                'options': 'dict',
                'domain_id': 'str'               # injected from auth (required)
                'workspace_id': 'str',           # injected from auth (required)
            }

        Returns:
            project_alert_config_vo (object)
        """

        project_id = params["project_id"]
        escalation_policy_id = params.get("escalation_policy_id")
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]

        project_alert_config_vo: ProjectAlertConfig = (
            self.project_alert_config_mgr.get_project_alert_config(
                project_id, domain_id, workspace_id
            )
        )

        if escalation_policy_id:
            escalation_policy_mgr: EscalationPolicyManager = self.locator.get_manager(
                "EscalationPolicyManager"
            )
            escalation_policy_vo = escalation_policy_mgr.get_escalation_policy(
                escalation_policy_id, domain_id, workspace_id
            )
            if (
                escalation_policy_vo.resource_group == "PROJECT"
                and escalation_policy_vo.project_id != project_id
            ):
                raise ERROR_INVALID_ESCALATION_POLICY(
                    escalation_policy_id=escalation_policy_id
                )

            params["escalation_policy"] = escalation_policy_vo

        if "options" in params:
            if "recovery_mode" not in params["options"]:
                params["options"][
                    "recovery_mode"
                ] = project_alert_config_vo.options.recovery_mode

            if "notification_urgency" not in params["options"]:
                params["options"][
                    "notification_urgency"
                ] = project_alert_config_vo.options.notification_urgency

        return self.project_alert_config_mgr.update_project_alert_config_by_vo(
            params, project_alert_config_vo
        )

    @transaction(
        permission="monitoring:ProjectAlertConfig.write",
        role_types=["WORKSPACE_MEMBER"],
    )
    @check_required(["project_id", "domain_id", "workspace_id"])
    def delete(self, params):
        """Delete project alert configuration

        Args:
            params (dict): {
                'project_id': 'str',       # required
                'domain_id': 'str'         # injected from auth (required)
                'workspace_id': 'str',     # injected from auth (required)
            }

        Returns:
            None
        """

        self.project_alert_config_mgr.delete_project_alert_config(
            params["project_id"], params["domain_id"], params["workspace_id"]
        )

    @transaction(
        permission="monitoring:ProjectAlertConfig.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["project_id", "domain_id", "workspace_id"])
    @change_only_key({"escalation_policy_info": "escalation_policy"})
    def get(self, params):
        """Get project alert configuration

        Args:
            params (dict): {
                'project_id': 'str',       # required
                'domain_id': 'str'         # injected from auth (required)
                'workspace_id': 'str',     # injected from auth (required)
            }

        Returns:
            project_alert_config_vo (object)
        """

        return self.project_alert_config_mgr.get_project_alert_config(
            params["project_id"], params["domain_id"], params["workspace_id"]
        )

    @transaction(
        permission="monitoring:ProjectAlertConfig.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["domain_id", "workspace_id"])
    @change_only_key(
        {"escalation_policy_info": "escalation_policy"}, key_path="query.only"
    )
    @append_query_filter(
        [
            "project_id",
            "escalation_policy_id",
            "domain_id",
            "workspace_id",
            "user_projects",
        ]
    )
    @append_keyword_filter(["project_id"])
    def list(self, params):
        """List project alert configurations

        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'project_id': 'str',
                'escalation_policy_id': 'str',
                'domain_id': 'str'         # injected from auth (required)
                'workspace_id': 'str',     # injected from auth (required)
                'user_projects': 'list'    # injected from auth (required)
            }

        Returns:
            project_alert_config_vos (object)
            total_count
        """

        query = params.get("query", {})
        return self.project_alert_config_mgr.list_project_alert_configs(query)

    @transaction(
        permission="monitoring:ProjectAlertConfig.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["query", "domain_id"])
    @append_query_filter(["domain_id", "workspace_id", "user_projects"])
    @append_keyword_filter(["project_id"])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'domain_id': 'str'         # injected from auth (required)
                'workspace_id': 'str',     # injected from auth (required)
                'user_projects': 'list'    # injected from auth (required)
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get("query", {})
        return self.project_alert_config_mgr.stat_project_alert_configs(query)



from spaceone.core.pygrpc.server import GRPCServer

from spaceone.monitoring.interface.grpc.alert import Alert
from spaceone.monitoring.interface.grpc.data_source import DataSource
from spaceone.monitoring.interface.grpc.escalation_policy import EscalationPolicy
from spaceone.monitoring.interface.grpc.event import Event
from spaceone.monitoring.interface.grpc.event_rule import EventRule
from spaceone.monitoring.interface.grpc.log import Log
from spaceone.monitoring.interface.grpc.metric import Metric
from spaceone.monitoring.interface.grpc.note import Note
from spaceone.monitoring.interface.grpc.project_alert_config import ProjectAlertConfig
from spaceone.monitoring.interface.grpc.webhook import Webhook

_all_ = ["app"]

app = GRPCServer()
app.add_service(Alert)
app.add_service(DataSource)
app.add_service(EscalationPolicy)
app.add_service(Event)
app.add_service(EventRule)
app.add_service(Log)
app.add_service(Metric)
app.add_service(Note)
app.add_service(ProjectAlertConfig)
app.add_service(Webhook)



import logging
from typing import Union
from spaceone.core.service import BaseService, transaction, convert_model
from spaceone.monitoring.plugin.webhook.model.webhook.request import (
    WebhookInitRequest,
    WebhookVerityRequest,
)
from spaceone.monitoring.plugin.webhook.model.webhook.response import WebhookResponse

_LOGGER = logging.getLogger(__name__)


class WebhookService(BaseService):
    @transaction
    @convert_model
    def init(self, params: WebhookInitRequest) -> Union[dict, WebhookResponse]:
        """init plugin by options

        :param params: WebhookRequest: {
            'options': 'dict'   # Required
        }

        :return:
            WebhookResponse: {
                'metadata': 'dict'
            }
        """
        func = self.get_plugin_method("init")
        response = func(params.dict())
        _LOGGER.debug(f"[WebhookService] init -> {response}")
        return WebhookResponse(**response)

    @transaction
    @convert_model
    def verify(self, params: WebhookVerityRequest) -> None:
        """verifying plugin

        :param params: WebhookRequest: {
                'options': 'dict'   # Request
            }

        :return:
            None
        """

        func = self.get_plugin_method("verify")
        func(params.dict())



import logging
from fastapi import Request, Body
from fastapi_utils.inferring_router import InferringRouter
from fastapi_utils.cbv import cbv
from spaceone.core.fastapi.api import BaseAPI, exception_handler
from spaceone.core.error import *
from spaceone.core.locator import Locator
from spaceone.monitoring.service import EventService
from spaceone.monitoring.model.event_model import CreateEvent

_LOGGER = logging.getLogger(__name__)

router = InferringRouter()


@cbv(router)
class Event(BaseAPI):
    @router.post("/webhook/{webhook_id}/{access_key}/events")
    async def create_event(self, access_key: str, webhook_id: str, request: Request):
        params, metadata = await self.parse_request(request)

        event_service: EventService = self.locator.get_service("EventService")
        event_service.create(
            {"webhook_id": webhook_id, "access_key": access_key, "data": params or {}}
        )
        return {}



WEBHOOK_DOMAIN = ""
CONSOLE_DOMAIN = ""

# Service Description
TITLE = "Documentation for Monitoring Webhook"
DESCRIPTION = ""

# Database Settings
DATABASE_AUTO_CREATE_INDEX = True
DATABASES = {
    "default": {
        "db": "monitoring",
        "host": "localhost",
        "port": 27017,
        "username": "",
        "password": "",
    }
}

# Cache Settings
CACHES = {
    "default": {},
    "local": {
        "backend": "spaceone.core.cache.local_cache.LocalCache",
        "max_size": 128,
        "ttl": 300,
    },
}

# Handler Settings
HANDLERS = {
    # "authentication": [{
    #     "backend": "spaceone.core.handler.authentication_handler:SpaceONEAuthenticationHandler"
    # }],
    # "authorization": [{
    #     "backend": "spaceone.core.handler.authorization_handler:SpaceONEAuthorizationHandler"
    # }],
    # "mutation": [{
    #     "backend": "spaceone.core.handler.mutation_handler:SpaceONEMutationHandler"
    # }],
    # "event": []
}

# Connector Settings
CONNECTORS = {
    # "SpaceConnector": {
    #     "backend": "spaceone.core.connector.space_connector:SpaceConnector",
    #     "endpoints": {
    #         "identity": "grpc://identity:50052",
    #         "inventory": "grpc://inventory:50051",
    #         "plugin": "grpc://plugin:50051",
    #         "repository": "grpc://repository:50051",
    #         "secret": "grpc://secret:50051",
    #         "notification": "grpc://notification:50051",
    #     },
    # },
    # "DataSourcePluginConnector": {},
    # "WebhookPluginConnector": {},
}

# Scheduler Settings
QUEUES = {}
SCHEDULERS = {}
WORKERS = {}
TOKEN = ""
TOKEN_INFO = {}

# Job Settings
JOB_TIMEOUT = 600

# Event Settings
SAME_EVENT_TIME = 600

INSTALLED_DATA_SOURCE_PLUGINS = [
    # {
    #     'name': '',
    #     'plugin_info': {
    #         'plugin_id': '',
    #         'version': '',
    #         'options': {},
    #         'secret_data': {},
    #         'schema': '',
    #         'upgrade_mode': ''
    #     },
    #     'tags':{
    #         'description': ''
    #     }
    # }
]

MAX_CONCURRENT_WORKER = 10
MAX_REQUEST_LIMIT = 200



import logging

from spaceone.core import config
from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector

_LOGGER = logging.getLogger(__name__)


class NotificationManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.notification_connector: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", service="notification", token=config.get_global("TOKEN")
        )

    def create_notification(self, message):
        _LOGGER.debug(f"Notify message: {message}")
        return self.notification_connector.dispatch("Notification.create", message)



import logging

from google.protobuf.json_format import MessageToDict

from spaceone.core.connector import BaseConnector
from spaceone.core import pygrpc
from spaceone.core.utils import parse_endpoint
from spaceone.core.error import *

__all__ = ["WebhookPluginConnector"]

_LOGGER = logging.getLogger(__name__)


class WebhookPluginConnector(BaseConnector):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.client = None

    def initialize(self, endpoint):
        static_endpoint = self.config.get("endpoint")

        if static_endpoint:
            endpoint = static_endpoint

        e = parse_endpoint(endpoint)
        self.client = pygrpc.client(
            endpoint=f'{e.get("hostname")}:{e.get("port")}', version="plugin"
        )

    def init(self, options):
        response = self.client.Webhook.init(
            {
                "options": options,
            },
            metadata=self.transaction.get_connection_meta(),
        )

        return self._change_message(response)

    def verify(self, options):
        params = {"options": options}

        self.client.Webhook.verify(
            params, metadata=self.transaction.get_connection_meta()
        )

    def parse_event(self, options, data):
        params = {"options": options, "data": data}

        response = self.client.Event.parse(
            params, metadata=self.transaction.get_connection_meta()
        )
        return self._change_message(response)

    @staticmethod
    def _change_message(message):
        return MessageToDict(message, preserving_proto_field_name=True)



import logging

from spaceone.core import cache
from spaceone.core.manager import BaseManager

from spaceone.monitoring.model.webhook_model import Webhook

_LOGGER = logging.getLogger(__name__)


class WebhookManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.webhook_model: Webhook = self.locator.get_model("Webhook")

    def create_webhook(self, params: dict) -> Webhook:
        def _rollback(vo: Webhook):
            _LOGGER.info(
                f"[create_webhook._rollback] "
                f"Delete webhook : {vo.name} "
                f"({vo.webhook_id})"
            )
            vo.delete()

        webhook_vo: Webhook = self.webhook_model.create(params)
        self.transaction.add_rollback(_rollback, webhook_vo)

        return webhook_vo

    def update_webhook(self, params):
        webhook_vo: Webhook = self.get_webhook(
            params["webhook_id"],
            params["domain_id"],
            params["workspace_id"],
            params.get("user_projects"),
        )
        return self.update_webhook_by_vo(params, webhook_vo)

    def update_webhook_by_vo(self, params: dict, webhook_vo: Webhook) -> Webhook:
        def _rollback(old_data: dict):
            _LOGGER.info(
                f"[update_webhook_by_vo._rollback] Revert Data : "
                f'{old_data["webhook_id"]}'
            )
            webhook_vo.update(old_data)

        self.transaction.add_rollback(_rollback, webhook_vo.to_dict())

        updated_vo: Webhook = webhook_vo.update(params)

        cache.delete(f"webhook-data:{updated_vo.webhook_id}")

        return updated_vo

    def delete_webhook(
        self,
        webhook_id: str,
        domain_id: str,
        workspace_id: str,
        user_projects: list = None,
    ):
        webhook_vo: Webhook = self.get_webhook(
            webhook_id, domain_id, workspace_id, user_projects
        )

        cache.delete(f"webhook-data:{webhook_vo.webhook_id}")

        webhook_vo.delete()

    def get_webhook(
        self,
        webhook_id: str,
        domain_id: str,
        workspace_id: str = None,
        user_projects: list = None,
    ) -> Webhook:
        conditions = {"webhook_id": webhook_id, "domain_id": domain_id}

        if workspace_id:
            conditions["workspace_id"] = workspace_id

        if user_projects:
            conditions["project_id"] = user_projects

        return self.webhook_model.get(**conditions)

    def get_webhook_by_id(self, webhook_id):
        return self.webhook_model.get(webhook_id=webhook_id)

    def list_webhooks(self, query: dict) -> dict:
        return self.webhook_model.query(**query)

    def stat_webhooks(self, query: dict) -> dict:
        return self.webhook_model.stat(**query)



from spaceone.core.error import *


class ERROR_NOT_SUPPORT_RESOURCE_TYPE(ERROR_INVALID_ARGUMENT):
    _message = (
        "Data source not support resource_type. (resource_type = {resource_type})"
    )


class ERROR_NOT_SUPPORT_METRIC_STAT(ERROR_INVALID_ARGUMENT):
    _message = "Data source not support metric stat. (stat = {stat})"


class ERROR_NOT_SUPPORT_PROVIDER_MONITORING(ERROR_INVALID_ARGUMENT):
    _message = "Data source not support provider on monitoring. (provider = {provider})"


class ERROR_NOT_MATCHING_RESOURCES(ERROR_INVALID_ARGUMENT):
    _message = "No Match found with given data. (monitoring = {monitoring})"


class ERROR_NOT_FOUND_REFERENCE_KEY(ERROR_INVALID_ARGUMENT):
    _message = (
        "Reference key not found in resource. (reference_keys = {reference_keys})"
    )


class ERROR_NOT_FOUND_REQUIRED_KEY(ERROR_INVALID_ARGUMENT):
    _message = (
        "Required key not found in plugin metadata. (data_source_id = {data_source_id})"
    )



import logging

from spaceone.core.service import *

from spaceone.monitoring.manager.alert_manager import AlertManager
from spaceone.monitoring.manager.note_manager import NoteManager
from spaceone.monitoring.model import Note

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class NoteService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.note_mgr: NoteManager = self.locator.get_manager("NoteManager")

    @transaction(
        permission="monitoring:Note.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["alert_id", "note", "domain_id", "workspace_id"])
    def create(self, params: dict) -> Note:
        """Create alert note

        Args:
            params (dict): {
                'alert_id': 'str',        # required
                'note': 'str',            # required
                'domain_id': 'str',       # injected from auth (required)
                'workspace_id': 'str',    # injected from auth (required)
            }

        Returns:
            note_vo (object)
        """

        alert_mgr: AlertManager = self.locator.get_manager("AlertManager")
        alert_vo = alert_mgr.get_alert(
            params["alert_id"], params["domain_id"], params["workspace_id"]
        )

        params["alert"] = alert_vo
        params["project_id"] = alert_vo.project_id
        params["created_by"] = self.transaction.get_meta("authorization.user_id")

        return self.note_mgr.create_note(params)

    @transaction(
        permission="monitoring:Note.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["note_id", "domain_id", "workspace_id"])
    def update(self, params: dict) -> Note:
        """Update alert note

        Args:
            params (dict): {
                'note_id': 'str',         # required
                'note': 'dict',
                'domain_id': 'str',       # injected from auth (required)
                'workspace_id': 'str',    # injected from auth (required)
                'user_projects': 'list',  # injected from auth
            }

        Returns:
            note_vo (object)
        """

        note_id = params["note_id"]
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]
        user_projects = params.get("user_projects")

        note_vo = self.note_mgr.get_note(
            note_id, domain_id, workspace_id, user_projects
        )

        # Check permission

        return self.note_mgr.update_note_by_vo(params, note_vo)

    @transaction(
        permission="monitoring:Note.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["note_id", "domain_id", "workspace_id"])
    def delete(self, params):
        """Delete alert note

        Args:
            params (dict): {
                'note_id': 'str',
                'domain_id': 'str',       # injected from auth (required)
                'workspace_id': 'str',    # injected from auth (required)
                'user_projects': 'list',  # injected from auth
            }

        Returns:
            None
        """

        self.note_mgr.delete_note(
            params["note_id"],
            params["domain_id"],
            params["workspace_id"],
            params.get("user_projects"),
        )

    @transaction(
        permission="monitoring:Note.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["note_id", "domain_id", "workspace_id"])
    def get(self, params):
        """Get alert note

        Args:
            params (dict): {
                'note_id': 'str',
                'domain_id': 'str',       # injected from auth (required)
                'workspace_id': 'str',    # injected from auth (required)
                'user_projects': 'list',  # injected from auth
            }

        Returns:
            note_vo (object)
        """

        return self.note_mgr.get_note(
            params["note_id"],
            params["domain_id"],
            params["workspace_id"],
            params.get("user_projects"),
        )

    @transaction(
        permission="monitoring:Note.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["domain_id"])
    @append_query_filter(
        [
            "note_id",
            "alert_id",
            "created_by",
            "project_id",
            "domain_id",
            "workspace_id",
            "user_projects",
        ]
    )
    @append_keyword_filter(["note_id", "note"])
    def list(self, params):
        """List alert notes

        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'note_id': 'str',
                'alert_id': 'str',
                'created_by': 'str',
                'workspace_id': 'str',
                'project_id': 'str',
                'domain_id': 'str',       # injected from auth (required)
                'user_projects': 'list',  # injected from auth
            }

        Returns:
            note_vos (object)
            total_count
        """

        query = params.get("query", {})
        return self.note_mgr.list_notes(query)

    @transaction(
        permission="monitoring:Note.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["query", "domain_id", "workspace_id"])
    @append_query_filter(["domain_id", "workspace_id", "user_projects"])
    @append_keyword_filter(["note_id", "note"])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'domain_id': 'str',          # injected from auth (required)
                'workspace_id': 'str',       # injected from auth (required)
                'user_projects': 'list',     # injected from auth
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get("query", {})
        return self.note_mgr.stat_notes(query)



from spaceone.core.error import *


class ERROR_INVALID_ESCALATION_POLICY_SCOPE(ERROR_INVALID_ARGUMENT):
    _message = "Only global escalation policy can be set by default. (escalation_policy_id = {escalation_policy_id})"


class ERROR_DEFAULT_ESCALATION_POLICY_NOT_ALLOW_DELETION(ERROR_INVALID_ARGUMENT):
    _message = "The default escalation policy does not allow deletion. (escalation_policy_id = {escalation_policy_id})"



from spaceone.api.monitoring.v1 import note_pb2, note_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Note(BaseAPI, note_pb2_grpc.NoteServicer):
    pb2 = note_pb2
    pb2_grpc = note_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("NoteService", metadata) as note_service:
            return self.locator.get_info("NoteInfo", note_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("NoteService", metadata) as note_service:
            return self.locator.get_info("NoteInfo", note_service.update(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("NoteService", metadata) as note_service:
            note_service.delete(params)
            return self.locator.get_info("EmptyInfo")

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("NoteService", metadata) as note_service:
            return self.locator.get_info("NoteInfo", note_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("NoteService", metadata) as note_service:
            note_vos, total_count = note_service.list(params)
            return self.locator.get_info(
                "NotesInfo", note_vos, total_count, minimal=self.get_minimal(params)
            )

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("NoteService", metadata) as note_service:
            return self.locator.get_info("StatisticsInfo", note_service.stat(params))



import datetime
import logging

from spaceone.core.service import *
from spaceone.core.utils import get_dict_value

from spaceone.monitoring.error import *
from spaceone.monitoring.manager.data_source_manager import DataSourceManager
from spaceone.monitoring.manager.data_source_plugin_manager import (
    DataSourcePluginManager,
)
from spaceone.monitoring.manager.identity_manager import IdentityManager
from spaceone.monitoring.manager.inventory_manager import InventoryManager
from spaceone.monitoring.manager.secret_manager import SecretManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class LogService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.identity_mgr: IdentityManager = self.locator.get_manager("IdentityManager")
        self.inventory_mgr: InventoryManager = self.locator.get_manager(
            "InventoryManager"
        )
        self.secret_mgr: SecretManager = self.locator.get_manager("SecretManager")
        self.data_source_mgr: DataSourceManager = self.locator.get_manager(
            "DataSourceManager"
        )
        self.ds_plugin_mgr: DataSourcePluginManager = self.locator.get_manager(
            "DataSourcePluginManager"
        )

    @transaction(
        permission="monitoring:Log.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["data_source_id", "resource_id", "start", "domain_id"])
    def list(self, params):
        """Get resource's logs

        Args:
            params (dict): {
                'data_source_id': 'str',   # required
                'resource_id': 'str',      # required
                'keyword': 'str',
                'start': 'datetime',       # required
                'end': 'datetime',         # default
                'sort': 'dict',
                'limit': 'int',
                'domain_id': 'str'
            }

        Returns:
            logs (list)
        """
        data_source_id = params["data_source_id"]
        resource_id = params["resource_id"]
        start = params["start"]
        end = params.get("end", str(datetime.datetime.now()))
        domain_id = params["domain_id"]
        secret_data = {}

        data_source_vo = self.data_source_mgr.get_data_source(data_source_id, domain_id)
        self._check_data_source_state(data_source_vo)
        self.plugin_initialize(data_source_vo)

        plugin_options = data_source_vo.plugin_info.options
        cloud_service_info = self.inventory_mgr.get_cloud_service(
            resource_id, domain_id
        )
        query = self.get_query_from_cloud_service(
            cloud_service_info, data_source_vo.plugin_info
        )
        secret = self.secret_mgr.get_secret_from_resource(
            cloud_service_info, data_source_vo, domain_id
        )
        secret_data = self.secret_mgr.get_secret_data(
            secret.get("secret_id", ""), domain_id
        )

        logs_info = self.ds_plugin_mgr.list_logs(
            secret.get("schema"),
            plugin_options,
            secret_data,
            query,
            params.get("keyword"),
            start,
            end,
            params.get("sort"),
            params.get("limit"),
        )

        return {"results": logs_info["results"], "domain_id": domain_id}

    def plugin_initialize(self, data_source_vo):
        endpoint = self.ds_plugin_mgr.get_data_source_plugin_endpoint_by_vo(
            data_source_vo
        )
        self.ds_plugin_mgr.initialize(endpoint)

    @staticmethod
    def get_query_from_cloud_service(cloud_service_info, plugin_info):
        metadata = plugin_info.metadata
        required_keys = metadata.get("required_keys", [])

        if required_keys:
            query_key = required_keys[0]
            query = get_dict_value(cloud_service_info, query_key, default_value={})
            if query:
                return query
            else:
                raise ERROR_REQUIRED_KEYS_NOT_EXISTS(plugin_id=plugin_info.plugin_id)
        else:
            raise ERROR_REQUIRED_KEYS_NOT_EXISTS(plugin_id=plugin_info.plugin_id)

    @staticmethod
    def _check_data_source_state(data_source_vo):
        if data_source_vo.state == "DISABLED":
            raise ERROR_DATA_SOURCE_STATE_DISABLED(
                data_source_id=data_source_vo.data_source_id
            )



import logging

from spaceone.core import cache
from spaceone.core.manager import BaseManager

from spaceone.monitoring.error.project_alert_config import *
from spaceone.monitoring.model.project_alert_config_model import ProjectAlertConfig

_LOGGER = logging.getLogger(__name__)


class ProjectAlertConfigManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.project_alert_config_model: ProjectAlertConfig = self.locator.get_model(
            "ProjectAlertConfig"
        )

    def create_project_alert_config(self, params: dict) -> ProjectAlertConfig:
        def _rollback(vo: ProjectAlertConfig):
            _LOGGER.info(
                f"[create_project_alert_config._rollback] "
                f"Delete project alert config : {vo.project_id}"
            )
            vo.delete()

        project_alert_config_vo: ProjectAlertConfig = (
            self.project_alert_config_model.create(params)
        )
        self.transaction.add_rollback(_rollback, project_alert_config_vo)

        return project_alert_config_vo

    def update_project_alert_config(self, params):
        project_alert_config_vo: ProjectAlertConfig = self.get_project_alert_config(
            params["project_id"], params["domain_id"], params["workspace_id"]
        )
        return self.update_project_alert_config_by_vo(params, project_alert_config_vo)

    def update_project_alert_config_by_vo(
        self, params: dict, project_alert_config_vo: ProjectAlertConfig
    ) -> ProjectAlertConfig:
        def _rollback(old_data: dict):
            _LOGGER.info(
                f"[update_project_alert_config_by_vo._rollback] Revert Data : "
                f'{old_data["project_id"]}'
            )
            project_alert_config_vo.update(old_data)

        self.transaction.add_rollback(_rollback, project_alert_config_vo.to_dict())

        updated_vo: ProjectAlertConfig = project_alert_config_vo.update(params)

        cache.delete(
            f"project-alert-options:{updated_vo.domain_id}:{updated_vo.project_id}"
        )
        cache.delete(
            f"escalation-policy-info:{updated_vo.domain_id}:{updated_vo.project_id}"
        )
        cache.delete(f"auto-recovery:{updated_vo.domain_id}:{updated_vo.project_id}")

        return updated_vo

    def delete_project_alert_config(
        self, project_id: str, domain_id: str, workspace_id: str
    ):
        project_alert_config_vo: ProjectAlertConfig = self.get_project_alert_config(
            project_id, domain_id, workspace_id
        )

        cache.delete(f"project-alert-options:{domain_id}:{project_id}")
        cache.delete(f"escalation-policy-info:{domain_id}:{project_id}")
        cache.delete(f"auto-recovery::{domain_id}:{project_id}")

        project_alert_config_vo.delete()

    def get_project_alert_config(
        self, project_id: str, domain_id: str, workspace_id: str
    ) -> ProjectAlertConfig:
        try:
            return self.project_alert_config_model.get(
                project_id=project_id, domain_id=domain_id, workspace_id=workspace_id
            )
        except ERROR_NOT_FOUND as e:
            raise ERROR_ALERT_FEATURE_IS_NOT_ACTIVATED(
                project_id=project_id, workspace_id=workspace_id
            )
        except Exception as e:
            raise e

    def list_project_alert_configs(self, query: dict) -> dict:
        return self.project_alert_config_model.query(**query)

    def stat_project_alert_configs(self, query: dict) -> dict:
        return self.project_alert_config_model.stat(**query)



import fnmatch
import logging

from spaceone.core.service import *

from spaceone.monitoring.error.event_rule import *
from spaceone.monitoring.manager.event_rule_manager import EventRuleManager
from spaceone.monitoring.manager.identity_manager import IdentityManager
from spaceone.monitoring.model.event_rule_model import EventRule

_LOGGER = logging.getLogger(__name__)

_SUPPORTED_CONDITION_KEYS = [
    "title",
    "description",
    "rule",
    "resource_id",
    "resource_name",
    "resource_type",
    "webhook_id",
    "project_id",
    "provider",
    "account",
    "additional_info.<key>",
]
_SUPPORTED_CONDITION_OPERATORS = ["eq", "contain", "not", "not_contain"]


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class EventRuleService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.event_rule_mgr: EventRuleManager = self.locator.get_manager(
            "EventRuleManager"
        )

    @transaction(
        permission="monitoring:EventRule.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(
        [
            "conditions",
            "conditions_policy",
            "actions",
            "resource_group",
            "domain_id",
            "workspace_id",
        ]
    )
    def create(self, params: dict) -> EventRule:
        """Create event rule

        Args:
            params (dict): {
                'name': 'str',
                'conditions': 'list',          # required
                'conditions_policy': 'str',    # required
                'actions': 'dict',             # required
                'options': 'dict',
                'project_id': 'str',
                'tags': 'dict',
                'resource_group': 'str',       # required
                'domain_id': 'str'             # injected from auth (required)
                'workspace_id': 'str'          # injected from auth (required)
            }

        Returns:
            event_rule_vo (object)
        """

        resource_group = params["resource_group"]
        project_id = params.get("project_id")
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]

        identity_mgr: IdentityManager = self.locator.get_manager("IdentityManager")

        if resource_group == "PROJECT":
            project_info = identity_mgr.get_project(project_id)
            params["workspace_id"] = project_info.get("workspace_id")
        else:
            identity_mgr.check_workspace(workspace_id, domain_id)
            params["project_id"] = "*"

        self._check_conditions(params["conditions"])
        self._check_actions(params["actions"])

        params["order"] = (
            self._get_highest_order(resource_group, project_id, domain_id, workspace_id)
            + 1
        )

        return self.event_rule_mgr.create_event_rule(params)

    @transaction(
        permission="monitoring:EventRule.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["event_rule_id", "domain_id", "workspace_id"])
    def update(self, params: dict) -> EventRule:
        """Update event rule

        Args:
            params (dict): {
                'event_rule_id': 'dict',        # required
                'name': 'str',
                'conditions': 'list',
                'conditions_policy': 'str',
                'actions': 'dict',
                'options': 'dict',
                'tags': 'dict',
                'domain_id': 'str',             # injected from auth (required)
                'workspace_id': 'str',          # injected from auth (required)
                'user_projects': 'list'         # injected from auth
            }

        Returns:
            event_rule_vo (object)
        """

        event_rule_id = params["event_rule_id"]
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]
        user_projects = params.get("user_projects")

        if "conditions" in params:
            self._check_conditions(params["conditions"])

        if "actions" in params:
            self._check_actions(params["actions"])

        event_rule_vo = self.event_rule_mgr.get_event_rule(
            event_rule_id, domain_id, workspace_id, user_projects
        )
        return self.event_rule_mgr.update_event_rule_by_vo(params, event_rule_vo)

    @transaction(
        permission="monitoring:EventRule.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["event_rule_id", "order", "domain_id", "workspace_id"])
    def change_order(self, params: dict) -> EventRule:
        """Get event rule

        Args:
            params (dict): {
                'event_rule_id': 'str',     # required
                'order': 'int',             # required
                'domain_id': 'str',         # injected from auth (required)
                'workspace_id': 'str',      # injected from auth (required)
                'user_projects': 'list'     # injected from auth
            }

        Returns:
            event_rule_vo (object)
        """

        event_rule_id = params["event_rule_id"]
        order = params["order"]
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]
        user_projects = params.get("user_projects")

        self._check_order(order)

        target_event_rule_vo = self.event_rule_mgr.get_event_rule(
            event_rule_id, domain_id, workspace_id, user_projects
        )

        if target_event_rule_vo.order == order:
            return target_event_rule_vo

        highest_order = self._get_highest_order(
            target_event_rule_vo.resource_group,
            target_event_rule_vo.project_id,
            target_event_rule_vo.domain_id,
            target_event_rule_vo.workspace_id,
        )

        if order > highest_order:
            raise ERROR_INVALID_PARAMETER(
                key="order",
                reason=f"There is no event rules greater than the {str(order)} order.",
            )

        event_rule_vos = self._get_all_event_rules(
            target_event_rule_vo.resource_group,
            target_event_rule_vo.project_id,
            domain_id,
            workspace_id,
            target_event_rule_vo.event_rule_id,
        )
        event_rule_vos.insert(order - 1, target_event_rule_vo)

        i = 0
        for event_rule_vo in event_rule_vos:
            if target_event_rule_vo != event_rule_vo:
                self.event_rule_mgr.update_event_rule_by_vo(
                    {"order": i + 1}, event_rule_vo
                )

            i += 1

        return self.event_rule_mgr.update_event_rule_by_vo(
            {"order": order}, target_event_rule_vo
        )

    @transaction(
        permission="monitoring:EventRule.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["event_rule_id", "domain_id", "workspace_id"])
    def delete(self, params):
        """Delete event rule

        Args:
            params (dict): {
                'event_rule_id': 'str',     # required
                'domain_id': 'str',         # injected from auth (required)
                'workspace_id': 'str',      # injected from auth (required)
                'user_projects': 'list'     # injected from auth
            }

        Returns:
            None
        """

        event_rule_id = params["event_rule_id"]
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]
        user_projects = params.get("user_projects")

        event_rule_vo: EventRule = self.event_rule_mgr.get_event_rule(
            event_rule_id, domain_id, workspace_id, user_projects
        )

        resource_group = event_rule_vo.resource_group
        project_id = event_rule_vo.project_id

        self.event_rule_mgr.delete_event_rule_by_vo(event_rule_vo)

        event_rule_vos = self._get_all_event_rules(
            resource_group, project_id, domain_id, workspace_id
        )

        i = 0
        for event_rule_vo in event_rule_vos:
            self.event_rule_mgr.update_event_rule_by_vo({"order": i + 1}, event_rule_vo)
            i += 1

    @transaction(
        permission="monitoring:EventRule.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "project_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @check_required(["event_rule_id", "domain_id", "workspace_id"])
    def get(self, params):
        """Get event rule

        Args:
            params (dict): {
                'event_rule_id': 'str',     # required
                'domain_id': 'str',         # injected from auth (required)
                'workspace_id': 'str',      # injected from auth
                'user_projects': 'list'     # injected from auth
            }

        Returns:
            event_rule_vo (object)
        """

        return self.event_rule_mgr.get_event_rule(
            params["event_rule_id"],
            params["domain_id"],
            params["workspace_id"],
            params.get("user_projects"),
        )

    @transaction(
        permission="monitoring:EventRule.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "project_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @check_required(["domain_id"])
    @append_query_filter(
        [
            "event_rule_id",
            "name",
            "resource_group",
            "project_id",
            "domain_id",
            "workspace_id",
            "user_projects",
        ]
    )
    @append_keyword_filter(["event_rule_id", "name"])
    def list(self, params):
        """List escalation polices

        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'event_rule_id': 'str',
                'name': 'str',
                'resource_group': 'str',
                'project_id': 'str',
                'workspace_id': 'str',
                'domain_id': 'str',         # injected from auth (required)
                'user_projects': 'list'     # injected from auth
            }

        Returns:
            event_rule_vos (object)
            total_count
        """

        query = params.get("query", {})
        return self.event_rule_mgr.list_event_rules(query)

    @transaction(
        permission="monitoring:EventRule.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "project_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @check_required(["query", "domain_id"])
    @append_query_filter(["domain_id", "workspace_id", "user_projects"])
    @append_keyword_filter(["event_rule_id", "name"])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'domain_id': 'str',         # injected from auth (required)
                'workspace_id': 'str',      # injected from auth
                'user_projects': 'list'     # injected from auth
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get("query", {})
        return self.event_rule_mgr.stat_event_rules(query)

    @staticmethod
    def _check_conditions(conditions: list) -> None:
        for condition in conditions:
            key = condition.get("key")
            value = condition.get("value")
            operator = condition.get("operator")

            if not (key and value and operator):
                raise ERROR_INVALID_PARAMETER(
                    key="conditions",
                    reason="Condition should have key, value and operator.",
                )

            if key not in _SUPPORTED_CONDITION_KEYS and not fnmatch.fnmatch(
                key, "additional_info.*"
            ):
                raise ERROR_INVALID_PARAMETER(
                    key="conditions.key",
                    reason=f"Unsupported key. "
                    f'({" | ".join(_SUPPORTED_CONDITION_KEYS)})',
                )
            if operator not in _SUPPORTED_CONDITION_OPERATORS:
                raise ERROR_INVALID_PARAMETER(
                    key="conditions.operator",
                    reason=f"Unsupported operator. "
                    f'({" | ".join(_SUPPORTED_CONDITION_OPERATORS)})',
                )

    @staticmethod
    def _check_actions(actions: dict) -> None:
        if "change_urgency" in actions:
            if actions["change_urgency"] not in ["HIGH", "LOW"]:
                raise ERROR_INVALID_PARAMETER(
                    key="actions.change_urgency",
                    reason=f"Unsupported urgency. (HIGH | LOW)",
                )

    @staticmethod
    def _check_order(order: int) -> None:
        if order <= 0:
            raise ERROR_INVALID_PARAMETER(
                key="order", reason="The order must be greater than 0."
            )

    def _get_highest_order(
        self, resource_group: str, project_id: str, domain_id: str, workspace_id: str
    ) -> int:
        query = {
            "filter": [
                {"k": "domain_id", "v": domain_id, "o": "eq"},
                {"k": "resource_group", "v": resource_group, "o": "eq"},
                {"k": "project_id", "v": project_id, "o": "eq"},
                {"k": "workspace_id", "v": workspace_id, "o": "eq"},
            ],
            "count_only": True,
        }

        event_rule_vos, total_count = self.event_rule_mgr.list_event_rules(query)
        return total_count

    def _get_all_event_rules(
        self,
        resource_group: str,
        project_id: str,
        domain_id: str,
        workspace_id: str,
        exclude_event_rule_id: str = None,
    ):
        query = {
            "filter": [
                {"k": "domain_id", "v": domain_id, "o": "eq"},
                {"k": "workspace_id", "v": workspace_id, "o": "eq"},
                {"k": "resource_group", "v": resource_group, "o": "eq"},
                {"k": "project_id", "v": project_id, "o": "eq"},
            ],
            "sort": {"key": "order"},
        }
        if exclude_event_rule_id is not None:
            query["filter"].append(
                {"k": "event_rule_id", "v": exclude_event_rule_id, "o": "not"}
            )

        event_rule_vos, total_count = self.event_rule_mgr.list_event_rules(query)
        return list(event_rule_vos)



from spaceone.monitoring.connector.datasource_plugin_connector import (
    DataSourcePluginConnector,
)
from spaceone.monitoring.connector.webhook_plugin_connector import (
    WebhookPluginConnector,
)



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class Error(EmbeddedDocument):
    error_code = StringField(max_length=128)
    message = StringField(max_length=2048)


class Job(MongoModel):
    job_id = StringField(max_length=40, generate_id="job", unique=True)
    domain_id = StringField(max_length=40, required=True)
    status = StringField(
        max_length=20,
        default="IN_PROGRESS",
        choices=("IN_PROGRESS", "SUCCESS", "ERROR", "TIMEOUT"),
    )
    errors = ListField(EmbeddedDocumentField(Error, default=None, null=True))
    total_tasks = IntField(default=0)
    remained_tasks = IntField(default=0)
    created_at = DateTimeField(auto_now_add=True)
    updated_at = DateTimeField(auto_now=True)
    finished_at = DateTimeField(default=None, null=True)

    meta = {
        "updatable_fields": [
            "status",
            "errors",
            "total_tasks",
            "remained_tasks",
            "finished_at",
        ],
        "ordering": ["-created_at"],
        "indexes": [
            # 'job_id',
            "domain_id",
            "status",
            "total_tasks",
            "remained_tasks",
            "created_at",
        ],
    }
